/***************************************************************************
    FIXES  var handling bugs
    Copyright (C) 2003 - 2004 by Thomas Hasart
    thasart@gmx.de
    
    Copyright (C) 2002 by Jakob Simon-Gaarde
    jsgaarde@tdcspace.dk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

 ***************************************************************************/

#include <qmessagebox.h>
#include "filepropertydlg.h"
#include <kmessagebox.h>
#include <kdebug.h>

//------------------------------------------------------------------------------------------------------------------------------------------
ScopeItem::ScopeItem(QListView *parent,const QString& aKey,const QString& aFile,TrollProjectItem* aItem)
		: QCheckListItem(parent,aItem->scopeString,QCheckListItem::CheckBox)
{
	myItem=aItem;
	myFileName=aFile;
	myKey=aKey;
	bool tMode= myItem->configuration.getSources(myKey,true).contains(myFileName);
	setOn(tMode);

}

//------------------------------------------------------------------------------------------------------------------------------------------
ScopeItem::ScopeItem(QCheckListItem *parent,const QString& aKey,const QString& aFile,TrollProjectItem* aItem)
		: QCheckListItem(parent,aItem->scopeString,QCheckListItem::CheckBox)
{
	myItem=aItem;
	myFileName=aFile;
	myKey=aKey;
	bool tMode= myItem->configuration.getSources(myKey,true).contains(myFileName);
	setOn(tMode);
}

//------------------------------------------------------------------------------------------------------------------------------------------
bool ScopeItem::isDirty()
{
	/*if (m_initialMode==isOn())
		return false;  // no change totally*/
	return true; // changed
}


//------------------------------------------------------------------------------------------------------------------------------------------
void ScopeItem::apply()
{
	if(!myItem)return;
	QStringList tList=myItem->configuration.getSources(myKey,true);
	tList.remove(myFileName);
	if(this->isOn())tList.append(myFileName);
	myItem->configuration.setSources(myKey,tList,true);
}

//------------------------------------------------------------------------------------------------------------------------------------------
FilePropertyDlg::FilePropertyDlg(TrollProjectItem *aPrjItem,int grtype, FileItem *aFileItem,
                                 QWidget *parent, const char* name, bool modal, WFlags fl)
		: FilePropertyBase(parent,name,modal,fl)
{
	if (grtype == GroupItem::InstallObject)
	{
		GroupItem* gitem = dynamic_cast<GroupItem*>(aFileItem->parent());
		
		if (gitem)	m_installObjectName = gitem->install_objectname;
	}
	
	m_gtype = grtype;
	myProjectItem = aPrjItem;
	myFileItem = aFileItem;
	ScopeTree->setRootIsDecorated(true);
	createScopeTree(myProjectItem);
}

//------------------------------------------------------------------------------------------------------------------------------------------
GroupItem* FilePropertyDlg::getInstallRoot(TrollProjectItem* aPrjtem)
{
	QPtrListIterator<GroupItem> it(aPrjtem->groups);
	for (;it.current();++it)
	{
		if ((*it)->groupType == GroupItem::InstallRoot)	return *it;
	}
	return 0;
}

//------------------------------------------------------------------------------------------------------------------------------------------
GroupItem* FilePropertyDlg::getInstallObject(TrollProjectItem* aPrjtem, const QString& objectname)
{
	GroupItem* instroot = getInstallRoot(aPrjtem);
	if (!instroot)	return 0;
	
	QPtrListIterator<GroupItem> it(instroot->installs);
	for (;it.current();++it)
	{
		if ((*it)->groupType == GroupItem::InstallObject &&
		        (*it)->install_objectname == objectname )
			return *it;
	}
	return 0;

}

//------------------------------------------------------------------------------------------------------------------------------------------
QString FilePropertyDlg::getKey()
{
	switch(m_gtype)
	{
		case GroupItem::Sources:		return ("SOURCES");
		case GroupItem::Headers:		return ("HEADERS");
		case GroupItem::Forms:		return ("FORMS");
		case GroupItem::Images:		return ("IMAGES");
		case GroupItem::IDLs:			return ("IDL_SOURCES");
		case GroupItem::Translations:	return ("TRANSLATIONS");
		case GroupItem::Yaccsources:	return ("YACCSOURCES");
		case GroupItem::Lexsources:	return ("LEXSOURCES");
		case GroupItem::Distfiles:		return ("DISTFILES");
		case GroupItem::InstallObject:	return ("INSTALLS");
		default: return("");
	}
}

//------------------------------------------------------------------------------------------------------------------------------------------
void FilePropertyDlg::createScopeTree(TrollProjectItem *aPrjtem,ScopeItem *aScopeItem)
{
	QPtrList<TrollProjectItem> tScopes=aPrjtem->scopes();
	QPtrListIterator<TrollProjectItem> spit(tScopes);
	for (; spit.current(); ++spit)
	{
		TrollProjectItem *nextSubproject = spit;
		QString tKey=getKey();

		ScopeItem *tItem;
		
		if (!aScopeItem)	tItem = new ScopeItem(ScopeTree,tKey,myFileItem->name,nextSubproject);
		else		tItem = new ScopeItem(aScopeItem,tKey,myFileItem->name,nextSubproject);
		
		if (!aScopeItem)	ScopeTree->insertItem(tItem);
		else		aScopeItem->insertItem(tItem);
		
		createScopeTree(nextSubproject,tItem);
	}
}


//------------------------------------------------------------------------------------------------------------------------------------------
void FilePropertyDlg::updateFileProperties()
{
	applyExcludedScopes();
	accept();
}

//------------------------------------------------------------------------------------------------------------------------------------------
void FilePropertyDlg::applyExcludedScopes(ScopeItem *aPrjtem)
{
	if (!aPrjtem)	aPrjtem = static_cast<ScopeItem*>(ScopeTree->firstChild());
	else			aPrjtem = static_cast<ScopeItem*>(aPrjtem->firstChild());
	
	while (aPrjtem)
	{
		aPrjtem->apply();
		applyExcludedScopes(aPrjtem);
		aPrjtem = static_cast<ScopeItem*>(aPrjtem->nextSibling());
	}
}

