/***************************************************************************
 *   Copyright (C) 2004 by Thomas Hasart
 *   thasart@gmx.de
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version
 *
 ***************************************************************************/


#include "tpconfigdialog.h"
#include "tpconfigdepspage.h"
#include "tpinsidelistview.h"
#include "tpoutsidelistview.h"
#include <kdebug.h>
#include <klistview.h>
#include <qlistview.h>
#include <kurlrequesterdlg.h> 
#include <kurlrequester.h> 
#include <kfile.h> 



//------------------------------------------------------------------------------------------------------------------------------------------
TPConfigDepsPage::TPConfigDepsPage( TPConfigDialog * aDlg, QWidget * parent, const char * name, WFlags fl )
		: TPConfigIncludePageBase(parent,name,fl)
{
	myDialog=aDlg;
	slotUpdateControls();
	//connect(myConf,SIGNAL(applyToConfig()),this,SLOT(applyToConfig()));
	myPchUrlRequester->hide();
}

//------------------------------------------------------------------------------------------------------------------------------------------
TPConfigDepsPage::~TPConfigDepsPage()
{
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDepsPage::slotUpdateControls( )
{
	QPtrList <TrollProjectItem> itemList=myDialog->project()->getAllProjects();
	myInsideListView->listView()->clear();
	myOutsideListView->listView()->clear();

	myInsideListView->listView()->setSorting(-1,false);
	myOutsideListView->listView()->setSorting(-1,false);

	QStringList depsList=myDialog->project()->configuration.m_prjdeps;

	QStringList::Iterator it=depsList.begin();
	for(;it!=depsList.end();++it)
	{
		TrollProjectItem *item=itemList.first();
		while(item)
		{
			if(item->type()==TrollBaseItem::Subproject)
			{
				if( (item->configuration.m_template==QTMP_LIBRARY )
				        || (item->configuration.m_template==QTMP_APPLICATION ) )
				{
					TrollProjectItem *sItem=item;
					if (sItem!=myDialog->project())
					{
						QString tmpLib;
						if(sItem->configuration.m_requirements & QD_SHARED)
							tmpLib = sItem->getSharedLibAddObject(myDialog->project()->getDownDirs());
						else if(sItem->configuration.m_requirements & QD_STATIC)
							tmpLib=sItem->getLibAddObject(myDialog->project()->getDownDirs());
						else
							tmpLib=sItem->getApplicationObject(myDialog->project()->getDownDirs());
						if(tmpLib==(*it))
						{
							InsideCheckListItem *newItem=new InsideCheckListItem(myInsideListView->listView(),myInsideListView->listView()->lastItem(),sItem,NULL);
							depsList.remove(it);
							it=depsList.begin();
							newItem->setOn(true);
							itemList.remove(item);
							item=itemList.first();
						}

					}
				}
			}
			item=itemList.next();
		}
	}


	//add all other prj in itemList unchecked
	TrollProjectItem *item=itemList.first();
	while(item)
	{
		if(item->type()==TrollBaseItem::Subproject)
		{
			if( (item->configuration.m_template==QTMP_LIBRARY )
			        || (item->configuration.m_template==QTMP_APPLICATION ) )
			{
				TrollProjectItem *sItem=item;
				if ((!sItem->isScope()) && (sItem!=myDialog->project()))
				{
					QString tmpLib;
					if(sItem->configuration.m_requirements & QD_SHARED)
						tmpLib = sItem->getSharedLibAddObject(myDialog->project()->getDownDirs());
					else if(sItem->configuration.m_requirements & QD_STATIC)
						tmpLib=sItem->getLibAddObject(myDialog->project()->getDownDirs());
					else
						tmpLib=sItem->getApplicationObject(myDialog->project()->getDownDirs());
					InsideCheckListItem *newItem=new InsideCheckListItem(myInsideListView->listView(),myInsideListView->listView()->lastItem(),sItem,NULL);
					newItem->setOn(false);
				}
			}
		}
		item=itemList.next();
	}

	QStringList::Iterator it1=depsList.begin();
	for(;it1!=depsList.end();++it1)
	{
		new QListViewItem(myOutsideListView->listView(),myOutsideListView->listView()->lastItem(),(*it1));
	}

}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDepsPage::applyToConfig( )
{
	//external project dependencies
	myDialog->project()->configuration.m_prjdeps.clear();
	QListViewItem *depItem=myOutsideListView->listView()->firstChild();
	while(depItem)
	{
		myDialog->project()->configuration.m_prjdeps.append(depItem->text(0));
		depItem=depItem->itemBelow();
	}
	
	//internal project dependencies
	InsideCheckListItem *insideItem=dynamic_cast<InsideCheckListItem *>(myInsideListView->listView()->firstChild());
	while(insideItem)
	{
		if (insideItem->isOn())
		{
			if(insideItem->prjItem->configuration.m_requirements & QD_STATIC)
			{
				QString tmpLib=insideItem->prjItem->getLibAddObject(myDialog->project()->getDownDirs());
				myDialog->project()->configuration.m_prjdeps.append(tmpLib);
			}
			else if(insideItem->prjItem->configuration.m_requirements & QD_SHARED)
			{
				QString tmpLib=insideItem->prjItem->getSharedLibAddObject(myDialog->project()->getDownDirs());
				myDialog->project()->configuration.m_prjdeps.append(tmpLib);
			}
			else
			{
				QString tmpApp=insideItem->prjItem->getApplicationObject(myDialog->project()->getDownDirs());
				myDialog->project()->configuration.m_prjdeps.append(tmpApp);
			}
		}
		insideItem=dynamic_cast<InsideCheckListItem *>(insideItem->itemBelow());
	}

}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDepsPage::slotOutsideEdit( )
{
    QListViewItem *item=myOutsideListView->listView()->currentItem();
    if(item==NULL)return;
    QString text=item->text(0);

    KURLRequesterDlg dialog(text, i18n("Change target:"), 0, 0);
    dialog.urlRequester()->setMode(KFile::File);
    if (dialog.exec() != QDialog::Accepted)
        return;
    QString path = dialog.urlRequester()->url();
    if (!path.isEmpty())
        item->setText(0, path);

}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDepsPage::slotOutsideAdd( )
{
    KURLRequesterDlg dialog("", i18n("Add target:"), 0, 0);
    dialog.urlRequester()->setMode(KFile::File);
    if (dialog.exec() != QDialog::Accepted)
        return;
    QString path = dialog.urlRequester()->url();
    if (!path.isEmpty())
        new QListViewItem(myOutsideListView->listView(), path);

}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDepsPage::slotAddLibDep( TrollProjectItem * aItem)
{
	QListViewItem *depItem=myInsideListView->listView()->firstChild();
	while(depItem)
	{
		if(	((InsideCheckListItem*)depItem)->prjItem==aItem)
		{
			((InsideCheckListItem*)depItem)->setOn(true);
		}
		depItem=depItem->itemBelow();
	}
}



#include "tpconfigdepspage.moc"

