/***************************************************************************
 *   Copyright (C) 2004 by Thomas Hasart
 *   thasart@gmx.de
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version
 *
 ***************************************************************************/

#include "tpconfigdialog.h"

//kde
#include <klocale.h>
#include <kiconloader.h>

//qt
#include <qlayout.h>

//pages
#include "tpconfigbasicpage.h"
#include "tpconfigbuildmodepage.h"
#include "tpconfigincludepage.h"
#include "tpconfiglibspage.h"
#include "tpconfiglibpathspage.h"
#include "tpconfigdepspage.h"
#include "tpconfigbuildorderpage.h"
#include "tpconfigcompilerpage.h"
#include "tpconfiglocationspage.h"
#include "tpconfigcorbapage.h"


#include "trollprojectconfig.h"
#include <kdebug.h>

//------------------------------------------------------------------------------------------------------------------------------------------
TPConfigDialog::TPConfigDialog( 	TrollProjectItem* aPrj, QWidget * parent, const char * name, bool modal )
 :KDialogBase( TreeList, i18n("Options"), Help|Apply|Ok|Cancel,Ok, parent, name, modal, true )
{
	
	myPrj=aPrj;
	setupBasicPage();
	setupBuildModePage();
	setupIncludePage();
	setupLibsPage();
	setupLibPathsPage();
	setupDepsPage();
	setupBuildOrderPage();
	setupCompilerPage();
	setupLocationsPage();
	setupCorbaPage();
	
	
	connect(this,SIGNAL(aboutToShowPage(QWidget* )),this,SLOT(applyToConfig()));
	setCaption("SubProject   "+myPrj->text(0));
	
	connect(myLibsPage,SIGNAL(libAdded(TrollProjectItem* )),myDepsPage,SLOT(slotAddLibDep(TrollProjectItem*)));
}


//------------------------------------------------------------------------------------------------------------------------------------------
TPConfigDialog::~TPConfigDialog()
{
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::setupBasicPage( )
{
	QFrame *page = addPage( i18n("Basic"), i18n("QMake Basic settings"),SmallIcon("spell") );
 	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 	if( topLayout == 0 ) { return; }
 	myBasicPage = new TPConfigBasicPage(this,page, "TPConfigBasicPage");
 	topLayout->addWidget( myBasicPage );
 	topLayout->addStretch(10);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::setupBuildModePage( )
{
	QFrame *page = addPage( i18n("Build Mode"), i18n("QMake Build Mode settings"),SmallIcon("spell") );
 	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 	if( topLayout == 0 ) { return; }
 	myBuildModePage = new TPConfigBuildModePage(this, page, "TPConfigBuildModePage");
 	topLayout->addWidget( myBuildModePage );
 	topLayout->addStretch(10);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::setupIncludePage( )
{
	QFrame *page = addPage( i18n("Include"), i18n("QMake Include settings"),SmallIcon("spell") );
 	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 	if( topLayout == 0 ) { return; }
 	myIncludePage = new TPConfigIncludePage(this, page, "TPConfigIncludePage");
 	topLayout->addWidget( myIncludePage );
 	topLayout->addStretch(10);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::slotDefault( )
{
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::slotOk( )
{
	applyToConfig();
	okClicked();
	accept();
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::slotApply( )
{
	applyToConfig();
	applyClicked();
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::setupLibsPage( )
{
	QFrame *page = addPage( i18n("Libs"), i18n("QMake Library settings"),SmallIcon("spell") );
 	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 	if( topLayout == 0 ) { return; }
 	myLibsPage = new TPConfigLibsPage(this, page, "TPConfigLibsPage");
 	topLayout->addWidget( myLibsPage );
 	topLayout->addStretch(10);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::setupDepsPage( )
{
	QFrame *page = addPage( i18n("Deps"), i18n("QMake Deps settings"),SmallIcon("spell") );
 	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 	if( topLayout == 0 ) { return; }
 	myDepsPage = new TPConfigDepsPage(this, page, "TPConfigDepsPage");
 	topLayout->addWidget( myDepsPage );
 	topLayout->addStretch(10);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::setupLibPathsPage( )
{
	QFrame *page = addPage( i18n("LibPaths"), i18n("QMake LibraryPaths settings"),SmallIcon("spell") );
 	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 	if( topLayout == 0 ) { return; }
 	myLibPathsPage = new TPConfigLibPathsPage(this, page, "TPConfigLibPathsPage");
 	topLayout->addWidget( myLibPathsPage );
 	topLayout->addStretch(10);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::setupBuildOrderPage( )
{
	QFrame *page = addPage( i18n("Build Order"), i18n("QMake Build Order settings"),SmallIcon("spell") );
 	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 	if( topLayout == 0 ) { return; }
 	myBuildOrderPage = new TPConfigBuildOrderPage(this, page, "TPConfigBuildOrderPage");
 	topLayout->addWidget( myBuildOrderPage );
 	topLayout->addStretch(10);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::setupCompilerPage( )
{
	QFrame *page = addPage( i18n("Compiler defines"), i18n("QMake Compiler defines"),SmallIcon("spell") );
 	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 	if( topLayout == 0 ) { return; }
 	myCompilerPage = new TPConfigCompilerPage(this, page, "TPConfigCompilerPage");
 	topLayout->addWidget( myCompilerPage );
 	topLayout->addStretch(10);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::setupLocationsPage( )
{
	QFrame *page = addPage( i18n("Locations"), i18n("QMake Locations"),SmallIcon("spell") );
 	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 	if( topLayout == 0 ) { return; }
 	myLocationsPage = new TPConfigLocationsPage(this, page, "TPConfigLocationsPage");
 	topLayout->addWidget( myLocationsPage );
 	topLayout->addStretch(10);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::applyToConfig( )
{

	myBasicPage->applyToConfig();
	myBuildModePage->applyToConfig();
	myIncludePage->applyToConfig();
	myLibsPage->applyToConfig();
	myLibPathsPage->applyToConfig();
	myDepsPage->applyToConfig();
	myBuildOrderPage->applyToConfig();
	myCompilerPage->applyToConfig();
	myLocationsPage->applyToConfig();
	myCorbaPage->applyToConfig();
	updateControls();
	
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::updateControls( )
{
	myBasicPage->slotUpdateControls();
	myBuildModePage->slotUpdateControls();
	myIncludePage->slotUpdateControls();
	myLibsPage->slotUpdateControls();
	myLibPathsPage->slotUpdateControls();
	myDepsPage->slotUpdateControls();
	myBuildOrderPage->slotUpdateControls();
	myCompilerPage->slotUpdateControls();
	myLocationsPage->slotUpdateControls();
	myCorbaPage->slotUpdateControls();
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::setProject( TrollProjectItem * aPrj )
{
	myPrj=aPrj;
	updateControls();
	setCaption("SubProject   "+myPrj->text(0));
}

//------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectItem * TPConfigDialog::project( )
{
	return(myPrj);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigDialog::setupCorbaPage( )
{
	QFrame *page = addPage( i18n("CORBA"), i18n("QMake CORBA"),SmallIcon("spell") );
 	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 	if( topLayout == 0 ) { return; }
 	myCorbaPage = new TPConfigCorbaPage(this, page, "TPConfigCorbaPage");
 	topLayout->addWidget( myCorbaPage );
 	topLayout->addStretch(10);

}


#include "tpconfigdialog.moc"
