/***************************************************************************
 *   Copyright (C) 2004 by Thomas Hasart
 *   thasart@gmx.de
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version
 *
 ***************************************************************************/


#include "tpconfigincludepage.h"
#include "tpinsidelistview.h"
#include "tpoutsidelistview.h"
#include "tpconfigdialog.h"
#include <qdir.h>
#include <kurlrequesterdlg.h>
#include <kurlrequester.h>
#include <kfile.h>


//------------------------------------------------------------------------------------------------------------------------------------------
TPConfigIncludePage::TPConfigIncludePage( TPConfigDialog *aDlg, QWidget * parent, const char * name, WFlags fl )
		: TPConfigIncludePageBase(parent,name,fl)
{
	myDialog=aDlg;
	slotUpdateControls();
	//connect(myConf,SIGNAL(applyToConfig()),this,SLOT(applyToConfig()));
	connect(myInsideListView->listView(),SIGNAL(clicked(QListViewItem* )),this,SLOT(slotInsideListViewClicked(QListViewItem*)));
}

//------------------------------------------------------------------------------------------------------------------------------------------
TPConfigIncludePage::~TPConfigIncludePage()
{}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigIncludePage::slotUpdateControls( )
{
	myInsideListView->listView()->clear();
	myOutsideListView->listView()->clear();
	myInsideListView->listView()->setSorting(-1,false);
	myOutsideListView->listView()->setSorting(-1,false);


	QStringList incList=myDialog->project()->configuration.m_incadd;
	QPtrList <TrollProjectItem> itemList=myDialog->project()->getAllProjects();
	TrollProjectItem *item=itemList.first();


	while(item)
	{
		if(item->type() == TrollBaseItem::Subproject)
		{
			//includes
			//temp strlist
			if( item->configuration.m_template==QTMP_LIBRARY ||
			        item->configuration.m_template==QTMP_APPLICATION)
			{
				TrollProjectItem *sItem=item;
				//if(sItem!=myProjectItem) //needed own path for other libs
				//if (!sItem->isScope)
				{
					QString tmpInc=sItem->getIncAddPath(myDialog->project()->getDownDirs());
					tmpInc=QDir::cleanDirPath(tmpInc);
					//InsideCheckListItem *newItem=new InsideCheckListItem(myInsideListView->listView(),myInsideListView->listView()->lastItem(),sItem,NULL);//, this);
					InsideCheckListItem *newItem=new InsideCheckListItem(myInsideListView->listView(),sItem,NULL);//, this);
					QStringList::Iterator it=incList.begin();
					for(;it!=incList.end();++it)
					{
						if((*it).find(tmpInc)>=0)
						{
							incList.remove(it);
							newItem->setOn(true);
							it=incList.begin();
						}
					}
				}
			}
		}
		//    item=(ProjectItem*)item->itemBelow();
		item=itemList.next();
	}

	//all other in incList are outside incs
	QStringList::Iterator it1=incList.begin();
	for(;it1!=incList.end();++it1)
	{
		new QListViewItem(myOutsideListView->listView(),myOutsideListView->listView()->lastItem(),(*it1));
	}
	myPchUrlRequester->setURL(myDialog->project()->configuration.m_pch);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigIncludePage::applyToConfig( )
{

	//add selected includes
	myDialog->project()->configuration.m_incadd.clear();
	myDialog->project()->configuration.m_incdeps.clear();

	InsideCheckListItem *insideItem=(InsideCheckListItem *)myInsideListView->listView()->firstChild();
	while(insideItem)
	{
		if(insideItem->isOn())
		{
			QString tmpInc=insideItem->prjItem->getIncAddPath(myDialog->project()->getDownDirs());
			tmpInc=QDir::cleanDirPath(tmpInc);
			if(!myDialog->project()->configuration.m_incadd.contains(tmpInc))
			{
				myDialog->project()->configuration.m_incadd.append(tmpInc);
			}
			if(!myDialog->project()->configuration.m_incdeps.contains(tmpInc))
			{
				myDialog->project()->configuration.m_incdeps.append(tmpInc);
			}
			
			if(!insideItem->prjItem->configuration.m_uipath.isEmpty())
			{
				tmpInc=QDir::cleanDirPath(tmpInc+"/"+insideItem->prjItem->configuration.m_uipath);
				if(!myDialog->project()->configuration.m_incadd.contains(tmpInc))
				{
					myDialog->project()->configuration.m_incadd.append(tmpInc);
				}
				//deps
				if(!myDialog->project()->configuration.m_incdeps.contains(tmpInc))
				{
					myDialog->project()->configuration.m_incdeps.append(tmpInc);
				}
			}

		}
		insideItem=(InsideCheckListItem*)insideItem->itemBelow();
	}

	QCheckListItem *outsideItem=(QCheckListItem *)myOutsideListView->listView()->firstChild();
	while(outsideItem)
	{
		if(!myDialog->project()->configuration.m_incadd.contains(outsideItem->text(0)))
		{
			myDialog->project()->configuration.m_incadd.append(outsideItem->text(0));
		}
		if(!myDialog->project()->configuration.m_incdeps.contains(outsideItem->text(0)))
		{
			myDialog->project()->configuration.m_incdeps.append(outsideItem->text(0));
		}
		outsideItem=(QCheckListItem*)outsideItem->itemBelow();
	}

	//PCH
	if(!myPchUrlRequester->url().isEmpty())
	{
		QFileInfo fi(myPchUrlRequester->url());
		myDialog->project()->configuration.m_pch=myPchUrlRequester->url();
	}
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigIncludePage::slotOutsideEdit( )
{
	QListViewItem *item=myOutsideListView->listView()->currentItem();
	if(item==NULL)return;
	QString text=item->text(0);

	KURLRequesterDlg dialog(text, i18n("Change include directory:"), 0, 0);
	dialog.urlRequester()->setMode(KFile::Directory);
	if (dialog.exec() != QDialog::Accepted)	return;
	
	QString dir = dialog.urlRequester()->url();
	if (!dir.isEmpty())	item->setText(0,dir);

}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigIncludePage::slotOutsideAdd( )
{
	KURLRequesterDlg dialog("", i18n("Add include directory:"), 0, 0);
	dialog.urlRequester()->setMode(KFile::Directory);
	if (dialog.exec() != QDialog::Accepted)	return;
	
	QString dir = dialog.urlRequester()->url();
	if (!dir.isEmpty())	new QListViewItem(myOutsideListView->listView(), dir);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigIncludePage::slotInsideListViewClicked( QListViewItem * aItem)
{
	if(!aItem)return;
	InsideCheckListItem *tItem=(InsideCheckListItem *)aItem;
	if(tItem->isOn())includeAdded(tItem->prjItem);
}

/*$SPECIALIZATION$*/


#include "tpconfigincludepage.moc"

