/***************************************************************************
 *   Copyright (C) 2004 by Thomas Hasart
 *   thasart@gmx.de
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version
 *
 ***************************************************************************/


#include "tpconfiglibspage.h"
#include "tpinsidelistview.h"
#include "tpoutsidelistview.h"
#include "tpconfigdialog.h"
#include <kdebug.h>
#include <klineeditdlg.h>
#include <kurlrequester.h>


//------------------------------------------------------------------------------------------------------------------------------------------
TPConfigLibsPage::TPConfigLibsPage( TPConfigDialog *aDlg, QWidget * parent, const char * name, WFlags fl )
		: TPConfigIncludePageBase(parent,name,fl)
{
	myDialog=aDlg;
	slotUpdateControls();
	//connect(myConf,SIGNAL(applyToConfig()),this,SLOT(applyToConfig()));
	connect(myInsideListView->listView(),SIGNAL(clicked(QListViewItem* )),this,SLOT(slotInsideListViewClicked(QListViewItem*)));
	myPchUrlRequester->hide();
	
}

//------------------------------------------------------------------------------------------------------------------------------------------
TPConfigLibsPage::~TPConfigLibsPage()
{
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigLibsPage::slotOutsideEdit()
{
    bool ok;
    QListViewItem *item=myOutsideListView->listView()->currentItem();
    if(item==NULL)return;
    QString text=item->text(0);

    QString dir = KLineEditDlg::getText(i18n("Change library to link:"), text, &ok, 0);
    if (ok && !dir.isEmpty() && dir != "-l")
        item->setText(0,dir);

}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigLibsPage::slotOutsideAdd()
{
    bool ok;
    QString dir = KLineEditDlg::getText(i18n("Add library to link:"), "-l", &ok, 0);
    if (ok && !dir.isEmpty() && dir != "-I")
        new QListViewItem(myOutsideListView->listView(), dir);

}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigLibsPage::slotUpdateControls( )
{
	QPtrList <TrollProjectItem> itemList=myDialog->project()->getAllProjects();

	myInsideListView->listView()->clear();
	myOutsideListView->listView()->clear();
	myInsideListView->listView()->setSorting(-1,false);
	myOutsideListView->listView()->setSorting(-1,false);

	//update librarys
	//temp strlist
	QStringList libList=myDialog->project()->configuration.m_libadd;


	QStringList::Iterator it=libList.begin();
	for(;it!=libList.end();++it)
	{
		TrollProjectItem *item=itemList.first();
		while(item)
		{
			if(item->type()==TrollBaseItem::Subproject)
			{

				//librarys
				if( item->configuration.m_template==QTMP_LIBRARY )
				{
					TrollProjectItem *sItem=(TrollProjectItem*)item;


					if(sItem!=myDialog->project())
					{
						// create lib string
						QString tmpLib=sItem->getLibAddObject(myDialog->project()->getDownDirs());
						if(sItem->configuration.m_requirements & QD_SHARED) tmpLib=/*"-l"+*/tmpLib;
						if(tmpLib==(*it))
						{
							InsideCheckListItem *newItem=new InsideCheckListItem(myInsideListView->listView(),myInsideListView->listView()->lastItem(),sItem,NULL);
							libList.remove(it);
							it=libList.begin();
							newItem->setOn(true);
							itemList.remove(item);
							item=itemList.first();
						}

					}
				}
			}
			item=itemList.next();
		}
	}


	//add all other prj in itemList unchecked
	TrollProjectItem *item=itemList.first();
	while(item)
	{
		if(item->type()==TrollBaseItem::Subproject)
		{
			//librarys
			if( item->configuration.m_template==QTMP_LIBRARY )
			{
				TrollProjectItem *sItem=item;
				if(sItem!=myDialog->project())
				{
					// create lib string
					QString tmpLib=sItem->getLibAddObject(myDialog->project()->getDownDirs());
					if(sItem->configuration.m_requirements & QD_SHARED) tmpLib=/*"-l"+*/tmpLib;
					InsideCheckListItem *newItem=new InsideCheckListItem(myInsideListView->listView(),myInsideListView->listView()->lastItem(),sItem,NULL);//,this);
					newItem->setOn(false);
				}
			}
		}
		item=itemList.next();
	}

	//all other in libList are outside libs
	QStringList::Iterator it1=libList.begin();
	for(;it1!=libList.end();++it1)
	{
		new QListViewItem(myOutsideListView->listView(),myOutsideListView->listView()->lastItem(),(*it1));
	}

}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigLibsPage::applyToConfig( )
{
	//add libs to link
	myDialog->project()->configuration.m_libadd.clear();
	myDialog->project()->configuration.m_librarypath.clear();

	//inside libs to link
	//  myProjectItem->configuration.m_prjdeps.clear();
	InsideCheckListItem* insideItem=(InsideCheckListItem *)myInsideListView->listView()->firstChild();
	while(insideItem)
	{
		if(insideItem->isOn())
		{

			QString tmpLib=insideItem->prjItem->getLibAddObject(myDialog->project()->getDownDirs());
			if(insideItem->prjItem->configuration.m_requirements & QD_SHARED)
			{
				tmpLib=/*"-l"+*/tmpLib;

				//add path if shared lib is linked
				QString tmpPath=insideItem->prjItem->getLibAddPath(myDialog->project()->getDownDirs());
				if(tmpPath!="")
				{
					myDialog->project()->configuration.m_librarypath.append(/*"-L"+*/tmpPath);
				}
			}
			myDialog->project()->configuration.m_libadd.append(tmpLib);

		}
		insideItem=(InsideCheckListItem*)insideItem->itemBelow();
	}

	//outside libs to link
	QCheckListItem *outsideItem=(QCheckListItem *)myOutsideListView->listView()->firstChild();
	while(outsideItem)
	{
		myDialog->project()->configuration.m_libadd.append(outsideItem->text(0));
		outsideItem=(QCheckListItem*)outsideItem->itemBelow();
	}

}

//------------------------------------------------------------------------------------------------------------------------------------------
void TPConfigLibsPage::slotInsideListViewClicked( QListViewItem * aItem)
{
	if(!aItem)return;
	InsideCheckListItem *tItem=(InsideCheckListItem *)aItem;
	if(tItem->isOn())libAdded(tItem->prjItem);
}



#include "tpconfiglibspage.moc"

