/***************************************************************************
 *   Copyright (C) 2004 by Thomas Hasart
 *   thasart@gmx.de
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version
 *
 ***************************************************************************/

#include "trollprojectconfig.h"
#include "filebuffer.h"
#include <kdebug.h>
#include <qfileinfo.h>
#include "trollprojectitem.h"
#define VALUES_PER_ROW  1



//------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectConfig::TrollProjectConfig()
{
}


//------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectConfig::~TrollProjectConfig()
{
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectConfig::saveProjectConfig(FileBuffer *Buffer,TrollProjectItem* aItem)
{
	if(!Buffer)return;
	saveSources("SUBDIRS",Buffer);
	saveSources("SOURCES",Buffer);
	saveSources("HEADERS",Buffer);
	saveSources("LEXSOURCES",Buffer);
	saveSources("YACCSOURCES",Buffer);
	saveSources("IMAGES",Buffer);
	saveSources("DISTFILES",Buffer);
	saveSources("INSTALLS",Buffer);
	saveSources("TRANSLATIONS",Buffer);
	saveSources("IDL_SOURCES",Buffer);
	saveSources("FORMS",Buffer);

		
	// Template variable
	Buffer->removeValues("TEMPLATE");
	if (m_template == QTMP_APPLICATION)
		Buffer->setValues("TEMPLATE",QString("app"),FileBuffer::VSM_RESET);
	if (m_template == QTMP_LIBRARY)
	{
		Buffer->setValues("TEMPLATE",QString("lib"),FileBuffer::VSM_RESET);
		Buffer->removeValues("VERSION");
		Buffer->setValues("VERSION",m_libraryversion,FileBuffer::VSM_RESET);
	}
	if (m_template == QTMP_SUBDIRS)
		Buffer->setValues("TEMPLATE",QString("subdirs"),FileBuffer::VSM_RESET);

	
	
	
	Buffer->removeValues("IDLC.commands");
	if(!idlc_command.isEmpty())Buffer->setValues("IDLC.commands",idlc_command,FileBuffer::VSM_RESET);
	Buffer->removeValues("IDLC.output");
	if(!idlc_output.isEmpty())Buffer->setValues("IDLC.output",idlc_output,FileBuffer::VSM_RESET);



	Buffer->removeValues("IDLC.input");
	Buffer->removeValues("QMAKE_EXTRA_UNIX_COMPILERS");
	Buffer->removeValues("PRE_TARGETDEPS");
	
	if((*mySources.find("IDL_SOURCES")).count()>0)
	{
		Buffer->setValues("IDLC.input",QString("IDL_SOURCES"),FileBuffer::VSM_RESET);
		
		Buffer->setValues("QMAKE_EXTRA_UNIX_COMPILERS",QString("IDLC"),FileBuffer::VSM_RESET);
		
		Buffer->setValues("PRE_TARGETDEPS",QString("${IDL_SOURCES}"),FileBuffer::VSM_RESET);
	}

	// Config variable
	Buffer->removeValues("CONFIG");
	QStringList configList;
	if (m_buildMode == QBM_RELEASE)
		configList.append("release");
	else if (m_buildMode == QBM_DEBUG)
		configList.append("debug");
	if (m_warnings == QWARN_ON)
		configList.append("warn_on");
	else if (m_warnings == QWARN_OFF)
		configList.append("warn_off");
	if (m_requirements & QD_QT)
		configList.append("qt");
	if (m_requirements & QD_OPENGL)
		configList.append("opengl");
	if (m_requirements & QD_THREAD)
		configList.append("thread");
	if (m_requirements & QD_X11)
		configList.append("x11");
	if (m_requirements & QD_STATIC)
		configList.append("staticlib");
	if (m_requirements & QD_SHARED)
		configList.append("dll");
	if (m_requirements & QD_PLUGIN)
		configList.append("plugin");
	if (m_requirements & QD_EXCEPTIONS)
		configList.append("exceptions");
	if (m_requirements & QD_STL)
		configList.append("stl");
	if (m_requirements & QD_RTTI)
		configList.append("rtti");
	if (m_requirements & QD_ORDERED)
		configList.append("ordered");
	if (m_requirements & QD_PCH)
		configList.append("precompile_header");
	if (m_inheritconfig == true)
		Buffer->setValues("CONFIG",configList,FileBuffer::VSM_APPEND,VALUES_PER_ROW);
	else
		Buffer->setValues("CONFIG",configList,FileBuffer::VSM_RESET,VALUES_PER_ROW);

	//PCH
		
	Buffer->removeValues("PRECOMPILED_HEADER");
	if(!m_pch.isEmpty())
	{
		//FIXME change to use same pch output in the project
		/*
		QStringList tList;
		QFileInfo fi(m_pch);
		tList.append("!exists("+aItem->getTargetBaseName()+".gch)" );
		tList.append("{");
		tList.append("system( ln -s "+fi.dirPath()+" "+aItem->getTargetBaseName()+".gch)" );
		tList.append("}");
		Buffer->appendBufferText(tList);
		*/
		Buffer->setValues("PRECOMPILED_HEADER",m_pch,FileBuffer::VSM_RESET);
	}
	
	
	// Config strings
	Buffer->removeValues("DESTDIR");
	if (!m_destdir.simplifyWhiteSpace().isEmpty())
		Buffer->setValues("DESTDIR",QString(m_destdir),FileBuffer::VSM_RESET,VALUES_PER_ROW);
	Buffer->removeValues("TARGET");
	if (!m_target.simplifyWhiteSpace().isEmpty())
		Buffer->setValues("TARGET",QString(m_target),FileBuffer::VSM_RESET,VALUES_PER_ROW);
	
	Buffer->removeValues("DEPENDPATH");
	if (m_incdeps.count())
		Buffer->setValues("DEPENDPATH",m_includepath,FileBuffer::VSM_RESET,VALUES_PER_ROW);
	
	Buffer->removeValues("INCLUDEPATH");
	if (m_includepath.count())
		Buffer->setValues("INCLUDEPATH",m_includepath,FileBuffer::VSM_RESET,VALUES_PER_ROW);
	
	
	
	Buffer->removeValues("DEFINES");
	if (m_defines.count())
		Buffer->setValues("DEFINES",m_defines,FileBuffer::VSM_RESET,VALUES_PER_ROW);
	Buffer->removeValues("QMAKE_CXXFLAGS_DEBUG");
	if (m_cxxflags_debug.count())
		Buffer->setValues("QMAKE_CXXFLAGS_DEBUG",m_cxxflags_debug,FileBuffer::VSM_APPEND,VALUES_PER_ROW);
	Buffer->removeValues("QMAKE_CXXFLAGS_RELEASE");
	if (m_cxxflags_release.count())
		Buffer->setValues("QMAKE_CXXFLAGS_RELEASE",m_cxxflags_release,FileBuffer::VSM_APPEND,VALUES_PER_ROW);
	Buffer->removeValues("QMAKE_LFLAGS_DEBUG");
	if (m_lflags_debug.count())
		Buffer->setValues("QMAKE_LFLAGS_DEBUG",m_lflags_debug,FileBuffer::VSM_RESET,VALUES_PER_ROW);
	Buffer->removeValues("QMAKE_LFLAGS_RELEASE");
	if (m_lflags_release.count())
		Buffer->setValues("QMAKE_LFLAGS_RELEASE",m_lflags_release,FileBuffer::VSM_RESET,VALUES_PER_ROW);
	Buffer->removeValues("QMAKE_LIBDIR");
	if (m_librarypath.count())
		Buffer->setValues("QMAKE_LIBDIR",m_librarypath,FileBuffer::VSM_RESET,VALUES_PER_ROW);
	Buffer->removeValues("OBJECTS_DIR");
	if (!m_objectpath.simplifyWhiteSpace().isEmpty())
		Buffer->setValues("OBJECTS_DIR",QString(m_objectpath),FileBuffer::VSM_RESET,VALUES_PER_ROW);
	Buffer->removeValues("UI_DIR");
	if (!m_uipath.simplifyWhiteSpace().isEmpty())
		Buffer->setValues("UI_DIR",QString(m_uipath),FileBuffer::VSM_RESET,VALUES_PER_ROW);
	Buffer->removeValues("MOC_DIR");
	if (!m_mocpath.simplifyWhiteSpace().isEmpty())
		Buffer->setValues("MOC_DIR",QString(m_mocpath),FileBuffer::VSM_RESET,VALUES_PER_ROW);
	Buffer->removeValues("MAKEFILE");
	if (!m_makefile.simplifyWhiteSpace().isEmpty())
		Buffer->setValues("MAKEFILE", m_makefile,FileBuffer::VSM_RESET,VALUES_PER_ROW);



	Buffer->removeValues("INCLUDEPATH");
	if (m_incadd.count()>0)
		Buffer->setValues("INCLUDEPATH",m_incadd,FileBuffer::VSM_RESET,VALUES_PER_ROW);

	Buffer->removeValues("LIBS");
	if (m_libadd.count()>0)
		Buffer->setValues("LIBS",m_libadd,FileBuffer::VSM_APPEND,VALUES_PER_ROW);

	Buffer->removeValues("TARGETDEPS");
	//  if (m_libadd.count()>0)
	Buffer->setValues("TARGETDEPS",m_prjdeps,FileBuffer::VSM_APPEND,VALUES_PER_ROW);

	//updateInstallObjects(item,Buffer);

	Buffer->saveBuffer("# thasart");
	
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectConfig::parseFiles(const QString &aKey, FileBuffer *aBuffer)
{
	QStringList tFiles;
	QStringList tFilesExclude;
	
	aBuffer->getValues(aKey,tFiles,tFilesExclude);
	mySources.replace(aKey,tFiles);
	mySourcesExclude.replace(aKey,tFilesExclude);
	
	if(aKey=="INSTALLS")
	{
		//parse install objects
		QStringList::iterator it = tFiles.begin();
		for (;it!=tFiles.end();++it)
		{
			if ((*it)=="target") continue;
			
			InstallDataObject* tObj=new InstallDataObject;
			tObj->install_objectname=(*it);

			QStringList path,path_excl;
			aBuffer->getValues((*it)+".path",path,path_excl);
			
			if (!path.isEmpty())	tObj->install_path = path[0];
			if (!path_excl.isEmpty())	tObj->install_pathExclude = path_excl[0];
		
			aBuffer->getValues((*it)+".files",tObj->files,tObj->filesExclude);

			myInstalls.insert((*it),tObj);
		}
	
		it = tFilesExclude.begin();
		for (;it!=tFilesExclude.end();++it)
		{
			if ((*it)=="target") continue;
			
			InstallDataObject* tObj=new InstallDataObject;
			tObj->install_objectname=(*it);

			QStringList path,path_excl;
			aBuffer->getValues((*it)+".path",path,path_excl);
			
			if (!path.isEmpty())	tObj->install_path = path[0];
			if (!path_excl.isEmpty())	tObj->install_pathExclude = path_excl[0];
		
			aBuffer->getValues((*it)+".files",tObj->files,tObj->filesExclude);

			myInstallsExclude.insert((*it),tObj);
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectConfig::saveSources(const QString &aKey, FileBuffer *aBuffer)
{
	if(aKey!="INSTALLS")
	{
		QStringList tFiles=getSources(aKey);
		QStringList tFilesExclude=getSources(aKey,true);
		aBuffer->removeValues(aKey);
		if(tFiles.count()>0)aBuffer->setValues(aKey,tFiles,FileBuffer::VSM_APPEND,VALUES_PER_ROW);
		if(tFilesExclude.count()>0)aBuffer->setValues(aKey,tFilesExclude,FileBuffer::VSM_EXCLUDE,VALUES_PER_ROW);
		return;
	}
	
	
	QDictIterator <InstallDataObject> it(myInstalls);
	QStringList tInstalls;
	aBuffer->removeValues(aKey);
	for(;it.current();++it)
	{
		tInstalls.append(it.currentKey());
		aBuffer->removeValues(it.currentKey()+".files");
		if((*it)->files.count()>0)	aBuffer->setValues(it.currentKey()+".files",(*it)->files,FileBuffer::VSM_APPEND,VALUES_PER_ROW);
		if((*it)->filesExclude.count()>0)	aBuffer->setValues(it.currentKey()+".files",(*it)->files,FileBuffer::VSM_EXCLUDE,VALUES_PER_ROW);
		
		aBuffer->removeValues(it.currentKey()+".path");
		aBuffer->setValues(it.currentKey()+".path",(*it)->install_path,FileBuffer::VSM_APPEND,VALUES_PER_ROW);
	
	}
	if(tInstalls.count()>0)aBuffer->setValues(aKey,tInstalls,FileBuffer::VSM_APPEND,VALUES_PER_ROW);
	
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectConfig::parseSources(FileBuffer *aBuffer)
{
	parseFiles("SOURCES",aBuffer);
	parseFiles("HEADERS",aBuffer);
	parseFiles("LEXSOURCES",aBuffer);
	parseFiles("YACCSOURCES",aBuffer);
	parseFiles("IMAGES",aBuffer);
	parseFiles("DISTFILES",aBuffer);
	parseFiles("INSTALLS",aBuffer);
	parseFiles("TRANSLATIONS",aBuffer);
	parseFiles("IDL_SOURCES",aBuffer);
	parseFiles("FORMS",aBuffer);
	parseFiles("SUBDIRS",aBuffer);
}

//------------------------------------------------------------------------------------------------------------------------------------------
QStringList TrollProjectConfig::getSources(const QString &aKey,bool exclude)
{
	if(exclude)return(*mySourcesExclude.find(aKey));
	return(*mySources.find(aKey));
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectConfig::parse(FileBuffer *aBuffer)
{
	parseSources(aBuffer);
	QStringList minusListDummy;
	QStringList lst;

	// set defaults
	m_template = QTMP_APPLICATION;
	m_buildMode = QBM_RELEASE;
	m_warnings = QWARN_ON;
	m_requirements = 0;

	// retrieve the project configuration
	aBuffer->getValues("TEMPLATE",lst,minusListDummy);
	if (lst.count())
	{
		if (lst[0] == "app")
			m_template = QTMP_APPLICATION;
		if (lst[0] == "lib")
		{
			m_template = QTMP_LIBRARY;
		}
		if (lst[0] == "subdirs")
		{
			m_template = QTMP_SUBDIRS;
		}
	}
	aBuffer->getValues("VERSION",lst,minusListDummy);
	if(lst.count())
		m_libraryversion = lst[0];

	m_inheritconfig = true;
	QPtrList<FileBuffer::ValueSetMode> configvaluesetmodes;
	aBuffer->getVariableValueSetModes("CONFIG",configvaluesetmodes);

	FileBuffer::ValueSetMode* ConfigSetMode = NULL;
	for(ConfigSetMode = configvaluesetmodes.first(); ConfigSetMode; ConfigSetMode = configvaluesetmodes.next())
	{
		if(ConfigSetMode != NULL)
		{
			if(ConfigSetMode[0] == FileBuffer::VSM_RESET)
			{
				m_inheritconfig = false;
				break;
			}
		}
	}

	aBuffer->getValues("CONFIG",lst,minusListDummy);
	if (lst.count())
	{
		// config debug/release
		if (lst.find("debug")!=lst.end())
			m_buildMode = QBM_DEBUG;
		if (lst.find("release")!=lst.end())
			m_buildMode = QBM_RELEASE;
		// config warnings on/off
		if (lst.find("warn_on")!=lst.end())
			m_warnings = QWARN_ON;
		if (lst.find("warn_off")!=lst.end())
			m_warnings = QWARN_OFF;
		// config requerments
		if (lst.find("qt")!=lst.end())
			m_requirements += QD_QT;
		if (lst.find("x11")!=lst.end())
			m_requirements += QD_X11;
		if (lst.find("thread")!=lst.end())
			m_requirements += QD_THREAD;
		if (lst.find("opengl")!=lst.end())
			m_requirements += QD_OPENGL;
		// config lib mode
		if (lst.find("staticlib")!=lst.end())
			m_requirements += QD_STATIC;
		if (lst.find("dll")!=lst.end())
			m_requirements += QD_SHARED;
		if (lst.find("plugin")!=lst.end())
			m_requirements += QD_PLUGIN;
		if (lst.find("exceptions")!=lst.end())
			m_requirements += QD_EXCEPTIONS;
		if (lst.find("stl")!=lst.end())
			m_requirements += QD_STL;
		if (lst.find("rtti")!=lst.end())
			m_requirements += QD_RTTI;
		if (lst.find("ordered")!=lst.end())
			m_requirements += QD_ORDERED;
		if (lst.find("precompile_header")!=lst.end())
			m_requirements += QD_PCH;
	}
	
	aBuffer->getValues("DESTDIR",lst,minusListDummy);
	if (lst.count())
		m_destdir = lst[0];

	aBuffer->getValues("INCLUDEPATH",lst,minusListDummy);
	if (lst.count())
		m_incadd = lst;

	aBuffer->getValues("DEPENDPATH",lst,minusListDummy);
	if (lst.count())
		m_incdeps= lst;
	
	aBuffer->getValues("LIBS",lst,minusListDummy);
	if (lst.count())
		m_libadd = lst;

	aBuffer->getValues("TARGET",lst,minusListDummy);
	if (lst.count())
		m_target = lst[0];

	aBuffer->getValues("TARGETDEPS",lst,minusListDummy);
	if (lst.count())
		m_prjdeps = lst;
	aBuffer->getValues("DEFINES",lst,minusListDummy);
	m_defines = lst;
	aBuffer->getValues("QMAKE_CXXFLAGS_DEBUG",lst,minusListDummy);
	m_cxxflags_debug = lst;
	aBuffer->getValues("QMAKE_CXXFLAGS_RELEASE",lst,minusListDummy);
	m_cxxflags_release = lst;
	aBuffer->getValues("QMAKE_LFLAGS_DEBUG",lst,minusListDummy);
	m_lflags_debug = lst;
	aBuffer->getValues("QMAKE_LFLAGS_RELEASE",lst,minusListDummy);
	m_lflags_release = lst;
	aBuffer->getValues("QMAKE_LIBDIR",lst,minusListDummy);
	m_librarypath = lst;
	aBuffer->getValues("OBJECTS_DIR",lst,minusListDummy);
	if (lst.count())
		m_objectpath = lst[0];
	aBuffer->getValues("UI_DIR",lst,minusListDummy);
	if (lst.count())
		m_uipath = lst[0];
	aBuffer->getValues("MOC_DIR",lst,minusListDummy);
	if (lst.count())
		m_mocpath = lst[0];
	aBuffer->getValues("MAKEFILE",lst,minusListDummy);
	if (lst.count())
		m_makefile = lst[0];

	// Install path
	aBuffer->getValues("target.path",lst,minusListDummy);
	if (lst.count())
		m_target_install_path = lst[0];
	aBuffer->getValues("INSTALLS",lst,minusListDummy);
	if (lst.count())
	{
		QStringList::iterator it = lst.begin();
		for (;it!=lst.end();it++)
		{
			if (*it == "target")
			{
				m_target_install = true;
				break;
			}
		}
	}

	aBuffer->getValues("IDLC.commands",lst,minusListDummy);
	if (lst.count())	this->idlc_command= lst.join(" ");

	aBuffer->getValues("IDLC.output",lst,minusListDummy);
	if (lst.count())	this->idlc_output= lst[0];
	
	aBuffer->getValues("PRECOMPILED_HEADER",lst,minusListDummy);
	if (lst.count())	this->m_pch= lst[0];
}

//------------------------------------------------------------------------------------------------------------------------------------------
QStringList TrollProjectConfig::subdirs( )
{
	return(*mySources.find("SUBDIRS"));
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectConfig::setSubdirs( const QStringList& aList )
{
	setSources("SUBDIRS",aList);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectConfig::addFileToConfig(const KURL &aUrl,bool exclude)
{
	//get Group
	QFileInfo fi(aUrl.fileName());
	QString tGroup=getTypeByExtension(fi.extension());

	if(!tGroup.isEmpty())
	{
		QStringList tList=getSources(tGroup,exclude);
		if(!tList.contains(aUrl.fileName()))
		{
			tList.append(aUrl.fileName());
			setSources(tGroup,tList,exclude);
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectConfig::getTypeByExtension(const QString &ext)
{
	if (ext == "cpp" || ext == "cc" || ext == "c" || ext == "C" || ext == "c++" || ext == "cxx")
		return "SOURCES";
	else if (ext == "hpp" || ext == "h" || ext == "hxx" || ext == "hh" || ext == "h++" || ext == "H")
		return "HEADERS";
	else if (ext == "ui")
		return "FORMS";
	else if (ext == "idl")
		return "IDL_SOURCES";
	else if (ext == "l" || ext == "ll" || ext == "lxx" || ext == "l++" )
		return "LEXSOURCES";
	else if (ext == "y" || ext == "yy" || ext == "yxx" || ext == "y++" )
		return "YACCSOURCES";
	else if (ext == "ts")
		return "TRANSLATIONS";
	else if (ext == "jpg" || ext == "jpeg" || ext == "png"|| ext == "bmp"|| ext == "gif")
		return "IMAGES";
		
	return "";
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectConfig::setSources( const QString & aKey, const QStringList & tFiles,bool exclude )
{
	if(exclude)mySourcesExclude.replace(aKey,tFiles);
	else mySources.replace(aKey,tFiles);
}

//------------------------------------------------------------------------------------------------------------------------------------------
QStringList TrollProjectConfig::getTargetDeps( )
{
	return(m_prjdeps);
}

//------------------------------------------------------------------------------------------------------------------------------------------
QDict< InstallDataObject > * TrollProjectConfig::getInstalls( bool exclude )
{
	if(exclude) return(&myInstallsExclude);
	return(&myInstalls);
}

//------------------------------------------------------------------------------------------------------------------------------------------
InstallDataObject * TrollProjectConfig::getInstall(const QString& aKey, bool exclude )
{
	if(exclude) return(myInstallsExclude[aKey]);
	return(myInstalls[aKey]);
}

//------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectConfig::addInstallRoot( const QString & aName )
{
	if(myInstalls[aName])return(false);
	
	InstallDataObject *tObj=new InstallDataObject;
	myInstalls.insert(aName,tObj);
	return(true);
}

//-------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectConfig::removeInstallRoot( const QString & aName )
{
	myInstalls.setAutoDelete(true);
	myInstalls.remove(aName);
	return(true);
}







