/***************************************************************************
 *   Copyright (C) 2003 by Thomas Hasart
 *   thasart@gmx.de
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version
 *
 ***************************************************************************/

#ifndef TROLLPROJECTCONFIG_H
#define TROLLPROJECTCONFIG_H

#include <qstring.h>
#include <qstringlist.h>
#include <qmap.h>
#include <qdict.h>
#include <kurl.h>

class FileBuffer;
class TrollProjectItem;

enum QMakeBuildMode    {QBM_DEBUG,QBM_RELEASE};
enum QMakeWarnings     {QWARN_ON,QWARN_OFF};
enum QMakeDepends      {QD_QT=1,QD_OPENGL=2,QD_THREAD=4,QD_X11=8,QD_STATIC=16,
						QD_SHARED=32,QD_PLUGIN=64,QD_EXCEPTIONS=128,QD_STL=256,
						QD_RTTI=512, QD_ORDERED=1024,QD_PCH=2048};
enum QMakeTemplate     {QTMP_APPLICATION,QTMP_LIBRARY,QTMP_SUBDIRS};


class InstallDataObject
{
	public:
		QString             install_objectname;
		QString             install_path;
		QString             install_pathExclude;
		QStringList	 files;
		QStringList	 filesExclude;
};

/**
@author KDevelop Authors
*/
class TrollProjectConfig
{
	public:
		TrollProjectConfig();
		~TrollProjectConfig();
		
		void saveProjectConfig(FileBuffer *Buffer,TrollProjectItem* aItem);
		void parse(FileBuffer *aBuffer);
		void parseSources(FileBuffer *aBuffer);
		void parseFiles(const QString &aKey, FileBuffer *aBuffer);
		QStringList getSources(const QString &aKey,bool exclude=false);
		void setSources(const QString &aKey,const QStringList& files,bool exclude=false);
		
		void saveSources(const QString &aKey, FileBuffer *aBuffer);
		QStringList getTargetDeps();
		
		QStringList subdirs();
		void setSubdirs(const QStringList& aList);
		void addFileToConfig(const KURL &dest,bool exclude=false);
		QString getTypeByExtension(const QString &ext);
		QDict <InstallDataObject>* getInstalls(bool exclude=false);
		InstallDataObject* getInstall(const QString&,bool exclude=false);
		bool addInstallRoot(const QString& aName);
		bool removeInstallRoot(const QString& aName);
		
		
		
		QMakeTemplate   m_template;
		QMakeBuildMode  m_buildMode;
		QMakeWarnings   m_warnings;
		int             m_requirements;
		QString         m_destdir;
		QString         m_target;
		QString         m_subdirName;
		QStringList     m_includepath;
		QString         m_libraryversion;
		QStringList     m_librarypath;
		QString         m_objectpath;
		QString         m_uipath;
		QString         m_mocpath;
		QString         m_makefile;
		QString         m_pch;
		QStringList     m_defines;
		QStringList     m_cxxflags_debug;
		QStringList     m_cxxflags_release;
		QStringList     m_lflags_debug;
		QStringList     m_lflags_release;
		QStringList     m_libadd;
		QStringList     m_incadd;

		QString         idlc_command;
		QString         idlc_output;
		QStringList     m_prjdeps;
		QStringList     m_incdeps;
		bool            m_inheritconfig;
		bool            m_target_install;
		QString         m_target_install_path;
		
		
		QMap <QString,QStringList> mySources;
		QMap <QString,QStringList> mySourcesExclude;
		QDict <InstallDataObject> myInstalls;
		QDict <InstallDataObject> myInstallsExclude;
};


#endif
