/***************************************************************************
 *   Copyright (C) 2004 by Thomas Hasart
 *   thasart@gmx.de
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version
 *
 ***************************************************************************/


#include "trollprojectitem.h"
#include <qdir.h>
#include "urlutil.h"
#include "pathutil.h"
#include <kiconloader.h>
#include <kdebug.h>
#include "filebuffer.h"
#include <klistview.h>
#include <klocale.h>
#include <kurl.h>
#include <addfilesdialog.h>
#include <qcombobox.h>
#include <kio/netaccess.h>
#include <kio/job.h>
#include <qdict.h>
#include <qfont.h>
#include <qpainter.h>
#include <kmessagebox.h>

#define VALUES_PER_ROW  1




//-------------------------------------------------------------------------------------------------------------------------------------------

TrollBaseItem::TrollBaseItem(Type type, QListView *parent, const QString &text)
		: QListViewItem(parent, text), typ(type)
{
	myBold=false;
}


//-------------------------------------------------------------------------------------------------------------------------------------------

TrollBaseItem::TrollBaseItem(Type type, TrollBaseItem *parent, const QString &text)
		: QListViewItem(parent, text), typ(type)
{
	myBold=false;
}




//------------------------------------------------------------------------------------------------------------------------------------------
/*
 * Class TrollProjectItem
 */
TrollProjectItem::TrollProjectItem(QListView *parent, const QString &text, const QString &aPath)
		: TrollBaseItem(Subproject, parent, text)
{
	m_RootBuffer=NULL;
	myIsScope=false;
	configuration.m_template = QTMP_APPLICATION;

	QDir dir(aPath);
	QStringList l = dir.entryList("*.pro");

	pro_file = l.count()?l[0]:(text + ".pro");
	QString proname = aPath + "/" + pro_file;
	myProWatch=new ProFileWatcher(this,proname);
	myProWatch->startScan();


	m_FileBuffer.bufferFile(proname);
	m_FileBuffer.handleScopes();
	configuration.parse(&m_FileBuffer);
	parseScopes(&m_FileBuffer);
	configuration.m_subdirName=text;

	myPath=aPath;
	init();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectItem::TrollProjectItem(TrollProjectItem *parent, const QString &text, const QString &aPath)
		: TrollBaseItem(Subproject, parent, text)
{
	m_RootBuffer=NULL;
	myIsScope=false;
	configuration.m_template = QTMP_APPLICATION;

	QDir dir(aPath);
	QStringList l = dir.entryList("*.pro");

	pro_file = l.count()?l[0]:(text + ".pro");
	QString proname = aPath + "/" + pro_file;
	myProWatch=new ProFileWatcher(this,proname);
	myProWatch->startScan();

	configuration.m_subdirName=text;


	m_FileBuffer.bufferFile(proname);
	m_FileBuffer.handleScopes();
	configuration.parse(&m_FileBuffer);
	parseScopes(&m_FileBuffer);
	

	myPath=aPath;
	init();
}

//------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectItem::TrollProjectItem(TrollProjectItem *parent, const QString &text,const QString& aPath,FileBuffer *aScopeBuffer)
		: TrollBaseItem(Subproject, parent, text)
{
	m_RootBuffer=NULL;
	myPath=aPath;
	scopeString=text;
	myIsScope=true;
	configuration.parse(aScopeBuffer);
	parseScopes(aScopeBuffer);
	init();
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::parseScopes(FileBuffer *aBuffer)
{
	QStringList tList=aBuffer->getChildScopeNames();
	QStringList tScopeList=aBuffer->getAllScopeStrings(1);

	QStringList::Iterator it;
	for (it = tList.begin(); it != tList.end(); ++it)
	{
		new TrollProjectItem(this, (*it),path() ,aBuffer->getSubBuffer( (*it) ));
	}
}


//-------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectItem::~TrollProjectItem()
{}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::getRelativPath()
{
	if(this->parent()==NULL||this->parent()->parent()==NULL) return("/"+this->text(0));//configuration.m_subdirName);
	else return(((TrollProjectItem*)this->parent())->getRelativPath()+"/"+this->text(0));//configuration.m_subdirName);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::getDownDirs()
{
	TrollProjectItem* pItem=this;
	while (pItem->parent())
		pItem=(TrollProjectItem*)pItem->parent();
	return getRelativePath(QDir::cleanDirPath(myPath),QDir::cleanDirPath(pItem->path()));
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::getSharedLibAddObject(const QString &downDirs)
{
	if(configuration.m_requirements & QD_SHARED)
	{
		QString tmpPath;
		if(configuration.m_destdir!="")
		{
			if (QDir::isRelativePath(configuration.m_destdir))
				tmpPath=downDirs+this->getRelativPath()+"/"+configuration.m_destdir;
			else
				tmpPath=configuration.m_destdir;
		}
		else
		{
			tmpPath=downDirs+this->getRelativPath()+"/";
		}

		tmpPath=QDir::cleanDirPath(tmpPath);

		QString libString=tmpPath+"/"+getTargetName();
		return(libString);
	}
	return "";
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::getApplicationObject(const QString &downDirs )
{
	QString tmpPath;
	if(configuration.m_destdir!="")
	{
		if (QDir::isRelativePath(configuration.m_destdir))
			tmpPath=downDirs+this->getRelativPath()+"/"+configuration.m_destdir;
		else
			tmpPath=configuration.m_destdir;
	}
	else
	{
		tmpPath=downDirs+this->getRelativPath()+"/";
	}

	tmpPath=QDir::cleanDirPath(tmpPath);

	return(tmpPath + "/"+getTargetName());
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::getLibAddObject(const QString& downDirs)
{
	if(configuration.m_requirements & QD_SHARED)
	{
		if(configuration.m_target!="")
		{
			return("-l"+configuration.m_target);
		}
		else
		{
			if(isWin32Scope())return("-l"+this->subdir+".dll");
			else return("-l"+this->subdir);
		}
	}
	else if(configuration.m_requirements & QD_STATIC)
	{
		QString tmpPath;
		if(configuration.m_destdir!="")
		{
			if (QDir::isRelativePath(configuration.m_destdir))
				tmpPath=downDirs+this->getRelativPath()+"/"+configuration.m_destdir;
			else
				tmpPath=configuration.m_destdir;
		}
		else
		{
			tmpPath=downDirs+this->getRelativPath()+"/";
		}

		tmpPath=QDir::cleanDirPath(tmpPath);

		QString libString=tmpPath+"/"+getTargetName();
		return(libString);
	}

	return("");
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::getLibAddPath(const QString &downDirs)
{

	//PATH only add if shared lib
	if(!(configuration.m_requirements & QD_SHARED))return("");

	QString tmpPath;
	if(configuration.m_destdir!="")
	{
		if (QDir::isRelativePath(configuration.m_destdir))
			tmpPath=downDirs+this->getRelativPath()+"/"+configuration.m_destdir;
		else
			tmpPath=configuration.m_destdir;
	}
	else
	{
		tmpPath=downDirs+this->getRelativPath()+"/";
	}

	tmpPath=QDir::cleanDirPath(tmpPath);

	return(tmpPath);

}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::getIncAddPath(const QString &downDirs)
{
	QString tmpPath=downDirs+this->getRelativPath();
	tmpPath=QDir::cleanDirPath(tmpPath);

	return(tmpPath);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::init()
{
	// Handle "subdirs" project
	if (configuration.m_template == QTMP_SUBDIRS)
	{
		QStringList tList=configuration.getSources("SUBDIRS");
		QStringList::Iterator it;
		for (it = tList.begin(); it != tList.end(); ++it)
		{
			new TrollProjectItem(this, (*it),path()+"/"+(*it));
		}
	}
	updateIcon( );
}


/*
 * Class GroupItem
 */



//-------------------------------------------------------------------------------------------------------------------------------------------
GroupItem::GroupItem(QListView *lv, GroupType type, const QString &text, const QString &scopeString)
		: TrollBaseItem(Group, lv, text)
{
	this->scopeString = scopeString;
	groupType = type;
	files.setAutoDelete(true);
	setPixmap(0, SmallIcon("tar"));
}


//-------------------------------------------------------------------------------------------------------------------------------------------

GroupItem::GroupType GroupItem::groupTypeForExtension(const QString &ext)
{
	if (ext == "cpp" || ext == "cc" || ext == "c" || ext == "C" || ext == "c++" || ext == "cxx")
		return Sources;
	else if (ext == "hpp" || ext == "h" || ext == "hxx" || ext == "hh" || ext == "h++" || ext == "H")
		return Headers;
	else if (ext == "ui")
		return Forms;
	else if (ext == "idl")
		return IDLs;
	else if (ext == "l" || ext == "ll" || ext == "lxx" || ext == "l++" )
		return Lexsources;
	else if (ext == "y" || ext == "yy" || ext == "yxx" || ext == "y++" )
		return Yaccsources;
	else if (ext == "ts")
		return Translations;
	else
		return NoType;
}



/*
 * Class FileItem
 */


//-------------------------------------------------------------------------------------------------------------------------------------------
FileItem::FileItem(TrollBaseItem *lv, const QString &text, bool exclude/*=false*/)
		: TrollBaseItem(File, lv, text)
{
	// if excluded is set the file is excluded in the subproject/project.
	// by default excluded is set to false, thus file is included
	excluded = exclude;
	setPixmap(0, SmallIcon("document"));
	this->name=text;
	excluded=exclude;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::updateProjectFile()
{
	if(!m_RootBuffer)return;
	myProWatch->stopScan();
	configuration.saveProjectConfig(m_RootBuffer,this);
	m_RootBuffer->saveBuffer(getHeader());
	myProWatch->startScan();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::getHeader()
{
	QString tFormat = i18n(	"# File generated by kdevelop's qmake manager. \n"
	                        "# ------------------------------------------- \n"
	                        "# Subdir relative project main directory: %s\n"
	                        "# Target is %s %s\n");
	QString tHeader;
	tHeader.sprintf(tFormat.ascii(),
	                getRelativPath().ascii(),
	                getTemplateString().ascii(),
	                getTargetName().ascii());


	return(tHeader);

}



//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::updateIcon( )
{
	if(myIsScope)setPixmap(0, SmallIcon("qmake_scope"));
	else
	{
		switch(configuration.m_template)
		{
			case QTMP_APPLICATION:
				setPixmap(0, SmallIcon("qmake_app"));
				break;

			case QTMP_LIBRARY:
				setPixmap(0, SmallIcon("qmake_lib"));
				break;

			case QTMP_SUBDIRS:
				setPixmap(0, SmallIcon("qmake_sub"));
				break;
		}
	}
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::showSources( KListView * aListView )
{

	aListView->clear();
	showInstallObjects(aListView);
	showSources( aListView,"LEXSOURCES",GroupItem::Lexsources);
	showSources( aListView,"YACCSOURCES",GroupItem::Yaccsources);
	showSources( aListView,"IMAGES",GroupItem::Images);
	showSources( aListView,"DISTFILES",GroupItem::Distfiles);
	showSources( aListView,"TRANSLATIONS",GroupItem::Lexsources);
	showSources( aListView,"IDL_SOURCES",GroupItem::IDLs);

	showSources( aListView,"FORMS",GroupItem::Forms );
	showSources( aListView,"HEADERS",GroupItem::Headers );
	showSources( aListView,"SOURCES",GroupItem::Sources);



}


//-------------------------------------------------------------------------------------------------------------------------------------------

void TrollProjectItem::showSources( KListView * aListView , QString aKey, GroupItem::GroupType aType)
{
	QStringList tList=configuration.getSources(aKey);

	GroupItem *tGroupItem=new GroupItem(aListView,aType, aKey,"");
	QStringList::Iterator it;
	for (it = tList.begin(); it != tList.end(); ++it)
	{
		// laterFileItem tItem=new FileItem(QListView *lv, const QString &text,bool exclude=false);
		new FileItem(tGroupItem,(*it));
		//new TrollBaseItem(File, tGroupItem,(*it));
	}
	tGroupItem->sortChildItems(0,true);
	if(tList.count()>0)tGroupItem->setOpen(true);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectItem::buildable( )
{
	switch(configuration.m_template)
	{
		case QTMP_APPLICATION:
			return(true);
			break;

		case QTMP_LIBRARY:
			return(true);
			break;

		case QTMP_SUBDIRS:
			return(true);
			break;
	}
	return(false);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectItem::isSubDir( )
{
	return(configuration.m_template==QTMP_SUBDIRS);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QPtrList< TrollProjectItem > TrollProjectItem::getAllProjects( )
{

	QPtrList <TrollProjectItem> tList;
	TrollProjectItem *tItem=(TrollProjectItem *)this->listView()->firstChild();
	while(tItem)
	{
		tList.append(tItem);
		getAllSubProjects(tItem,&tList);
		tItem=(TrollProjectItem *)tItem->nextSibling();
	}
	return(tList);
}

//-------------------------------------------------------------------------------------------------------------------------------------------
QPtrList< TrollProjectItem > TrollProjectItem::scopes( )
{

	QPtrList <TrollProjectItem> tList;
	TrollProjectItem *tItem=(TrollProjectItem *)this->firstChild();
	while(tItem)
	{
		if(tItem->isScope())tList.append(tItem);
		tItem=(TrollProjectItem *)tItem->nextSibling();
	}
	return(tList);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::getAllSubProjects(TrollBaseItem *aItem, QPtrList <TrollProjectItem> *aList)
{

	TrollProjectItem *tItem=(TrollProjectItem *)aItem->firstChild();
	while(tItem)
	{
		aList->append(tItem);
		getAllSubProjects(tItem,aList);
		tItem=(TrollProjectItem *)tItem->nextSibling();
	}
}


//-------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectItem::runable( )
{
	return(configuration.m_template==QTMP_APPLICATION);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectItem::canHaveSources( )
{
	return(configuration.m_template!=QTMP_SUBDIRS);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectItem::isRoot( )
{
	return(this->parent()?true:false);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectItem::isScope( )
{
	return(myIsScope);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::path( )
{
	return(myPath);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectItem * TrollProjectItem::findFile(QString aAbsFileName)
{
	//find in sources
	QStringList tList=configuration.getSources("SOURCES");
	QFileInfo fi(aAbsFileName);
	for (QStringList::Iterator it = tList.begin(); it != tList.end(); ++it )
	{
		QFileInfo fi2(myPath, *it);
		if (fi.absFilePath() == fi2.absFilePath())
			return(this);
	}

	//find in headers
	tList=configuration.getSources("HEADERS");
	for (QStringList::Iterator it = tList.begin(); it != tList.end(); ++it )
	{
		QFileInfo fi2(myPath, *it);
		if (fi.absFilePath() == fi2.absFilePath())
			return(this);
	}

	//find in subprojects
	TrollProjectItem *tRetItem=NULL;
	TrollProjectItem *tItem=(TrollProjectItem*)this->firstChild();
	while(tItem && !tRetItem)
	{
		tRetItem=tItem->findFile(aAbsFileName);
		tItem=(TrollProjectItem*)tItem->nextSibling();
	}
	return(tRetItem);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QStringList TrollProjectItem::getAllFiles()
{
	QStringList tRetList;
	QStringList tList;

	tList=configuration.getSources("HEADERS");
	for (QStringList::Iterator it = tList.begin(); it != tList.end(); ++it )
	{
		tRetList+= (this->getRelativPath()+"/"+(*it)).mid(1);
	}

	tList=configuration.getSources("SOURCES");
	for (QStringList::Iterator it = tList.begin(); it != tList.end(); ++it )
	{
		tRetList+= (this->getRelativPath()+"/"+(*it)).mid(1);
	}
	
	tList=configuration.getSources("FORMS");
	for (QStringList::Iterator it = tList.begin(); it != tList.end(); ++it )
	{
		tRetList+= (this->getRelativPath()+"/"+(*it)).mid(1);
	}

	tList=configuration.getSources("LEXSOURCES");
	for (QStringList::Iterator it = tList.begin(); it != tList.end(); ++it )
	{
		tRetList+= (this->getRelativPath()+"/"+(*it)).mid(1);
	}
	
	tList=configuration.getSources("YACCSOURCES");
	for (QStringList::Iterator it = tList.begin(); it != tList.end(); ++it )
	{
		tRetList+= (this->getRelativPath()+"/"+(*it)).mid(1);
	}

	tList=configuration.getSources("IMAGES");
	for (QStringList::Iterator it = tList.begin(); it != tList.end(); ++it )
	{
		tRetList+= (this->getRelativPath()+"/"+(*it)).mid(1);
	}
		
	tList=configuration.getSources("DISTFILES");
	for (QStringList::Iterator it = tList.begin(); it != tList.end(); ++it )
	{
		tRetList+= (this->getRelativPath()+"/"+(*it)).mid(1);
	}
	
	tList=configuration.getSources("TRANSLATIONS");
	for (QStringList::Iterator it = tList.begin(); it != tList.end(); ++it )
	{
		tRetList+= (this->getRelativPath()+"/"+(*it)).mid(1);
	}
	
	tList=configuration.getSources("IDL_SOURCES");
	for (QStringList::Iterator it = tList.begin(); it != tList.end(); ++it )
	{
		tRetList+= (this->getRelativPath()+"/"+(*it)).mid(1);
	}
	
	
	tRetList+= this->getRelativPath()+"/"+pro_file;

	TrollProjectItem *tItem=(TrollProjectItem*)this->firstChild();
	while(tItem)
	{
		tRetList+= tItem->getAllFiles();
		tItem=(TrollProjectItem*)tItem->nextSibling();
	}
	return(tRetList);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::saveConfig( )
{
	myProWatch->stopScan();

	if(isScope())
	{
		((TrollProjectItem*)parent())->saveConfig();
	}
	else
	{
		//save projectConfig
		configuration.saveProjectConfig(&m_FileBuffer,this);

		//save scopes
		saveScopes(&m_FileBuffer);
		m_FileBuffer.saveBuffer(getHeader());
	}
	myProWatch->startScan();
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::saveScopes(FileBuffer *aBuffer )
{
	TrollProjectItem *tItem=(TrollProjectItem*)this->firstChild();
	while(tItem)
	{
		if(tItem->isScope())
		{
			tItem->saveScopes(aBuffer->getSubBuffer(tItem->text(0)));
		}
		tItem=(TrollProjectItem*)tItem->nextSibling();
	}
	if(this->isScope())
	{
		configuration.saveProjectConfig(aBuffer,this);
	}
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::readConfig( )
{
	m_FileBuffer.bufferFile(myPath + "/" + pro_file);
	configuration.parse(&m_FileBuffer);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QStringList TrollProjectItem::addFiles( )
{
	myProWatch->stopScan();
	QStringList tRetList;
	QString cleanDir = QDir::cleanDirPath(path());
	QString  filter = "*.cpp *.cc *.c *.hpp *.h *.ui|" + i18n("Source Files");
	filter += "\n*|" + i18n("All Files");
#if KDE_VERSION >= 310
	AddFilesDialog *dialog = new AddFilesDialog(cleanDir,
	                         filter,
	                         0,
	                         "Insert existing files",
	                         true, new QComboBox(false));
#else
	AddFilesDialog *dialog = new AddFilesDialog(cleanDir,
							 filter,
							 0,
							 i18n("Insert Existing Files").ascii(),
							 true);
#endif
	dialog->setMode(KFile::Files | KFile::ExistingOnly | KFile::LocalOnly);
	dialog->exec();
	KURL::List 	files = dialog->selectedURLs();
	switch (dialog->mode())
	{
		case AddFilesDialog::Copy:
			{
				// Copy selected files to current subproject folder


				KURL dest;
				KURL::List::iterator it;
				for ( it = files.begin(); it != files.end(); ++it )
				{
					dest.setPath(QDir::cleanDirPath(path() + "/" + (*it).fileName()));
					QFileInfo fi(QDir::cleanDirPath(path() + "/" + (*it).fileName()));
					if(!fi.exists())
					{
						if (KIO::NetAccess::file_copy((*it), dest))
						{
							tRetList+=(this->getRelativPath()+"/"+(*it).fileName()).mid(1);
						}
					}
					configuration.addFileToConfig(dest);
				}
			}
			break;

		case AddFilesDialog::Link:
			{
				KURL dest;
				KURL::List::iterator it;
				for ( it = files.begin(); it != files.end(); ++it )
				{
					dest.setPath(QDir::cleanDirPath(path() + "/" + (*it).fileName()));
					QFileInfo fi(QDir::cleanDirPath(path() + "/" + (*it).fileName()));
					if(!fi.exists())
					{
						KIO::link((*it), path());
						{
							tRetList+=(this->getRelativPath()+"/"+(*it).fileName()).mid(1);
						}
					}
					configuration.addFileToConfig(dest);
				}
			}
			break;

		case AddFilesDialog::Relative:
			{
				KURL dest;
				KURL::List::iterator it;
				for ( it = files.begin(); it != files.end(); ++it )
				{
					dest.setPath(QDir::cleanDirPath(path() + "/" + (*it).fileName()));
					QFileInfo fi(QDir::cleanDirPath(path() + "/" + (*it).fileName()));
					if(!fi.exists())
					{
						KIO::link((*it), path());
						{
							tRetList+=(this->getRelativPath()+"/"+(*it).fileName()).mid(1);
						}
					}
					configuration.addFileToConfig(dest);
				}
			}
			break;
	}
	saveConfig();
	myProWatch->startScan();
	return(tRetList);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QStringList TrollProjectItem::addFiles(const QStringList & aList)
{

	KURL dest;
	QStringList tRetList;
	QStringList::const_iterator it;
	for ( it = aList.begin(); it != aList.end(); ++it )
	{
		dest.setPath(QDir::cleanDirPath(path() + "/" + (*it)));
		configuration.addFileToConfig(dest);
		tRetList+=(this->getRelativPath()+"/"+(*it)).mid(1);
	}
	return(tRetList);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::addFile(const QString & aFileName)
{
	KURL dest;
	dest.setPath(QDir::cleanDirPath(path() + "/" + aFileName));
	configuration.addFileToConfig(dest);
	saveConfig();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::removeFile( const QString &aFileName )
{
	removeFile("SOURCES",aFileName);
	removeFile("HEADERS",aFileName);
	removeFile("LEXSOURCES",aFileName);
	removeFile("YACCSOURCES",aFileName);
	removeFile("IMAGES",aFileName);
	removeFile("DISTFILES",aFileName);
	removeFile("IDLS",aFileName);
	removeFile("FORMS",aFileName);
	removeFile("TRANSLATIONS",aFileName);
	if(isScope())
	{
		//if this is scope and this file is part of parent scope or project
		// 	file must added to files exclude
		if(getProject()->hasFile(aFileName))
		{
			configuration.addFileToConfig(aFileName,true);

		}
	}
	saveConfig();

	return( (getRelativPath()+"/"+aFileName).mid(1) );
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::removeFile(const QString &aKey, const QString& aFileName )
{
	QStringList tList=configuration.getSources(aKey);
	tList.remove(aFileName);
	configuration.setSources(aKey,tList);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::removeProject( )
{
	TrollProjectItem* tItem=(TrollProjectItem*)this->parent();
	if(tItem)
	{
		tItem->removeFile("SUBDIRS",this->text(0));
		tItem->saveConfig();
		tItem->takeItem(this);
	}
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::createScope( const QString & aName )
{
	if(this->isScope())
	{}
	else
	{
		m_FileBuffer.makeScope(aName);
		TrollProjectItem *tItem=new TrollProjectItem(this,aName,path(),m_FileBuffer.getSubBuffer(aName));
		tItem->saveConfig();
	}
}
//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::removeScope()
{
	if(!this->isScope())return;
	TrollProjectItem* tItem=(TrollProjectItem*)this->parent();
	if(tItem)
	{
		tItem->m_FileBuffer.removeScope(scopeString,scopeString);
		tItem->removeItem(this);
		tItem->saveConfig();
	}
}

//------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectItem * TrollProjectItem::getProject( )
{
	if(!isScope())return(this);

	TrollProjectItem* tItem=(TrollProjectItem*)this->parent();
	return(tItem->getProject());
}

//------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectItem::hasFile( const QString & aFileName )
{
	QStringList tList;
	tList=configuration.getSources("HEADERS");
	tList+=configuration.getSources("SOURCES");
	tList+=configuration.getSources("LEXSOURCES");
	tList+=configuration.getSources("YACCSOURCES");
	tList+=configuration.getSources("IMAGES");
	tList+=configuration.getSources("DISTFILES");
	tList+=configuration.getSources("IDLS");
	tList+=configuration.getSources("FORMS");
	if(tList.contains(aFileName))return(true);

	if(isScope())
	{
		TrollProjectItem *tItem=(TrollProjectItem*)this->firstChild();
		while(tItem)
		{
			if(tItem->isScope())
			{
				if(hasFile(aFileName ))return(true);
			}
			tItem=(TrollProjectItem*)tItem->nextSibling();
		}
	}
	return(false);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::addSubDir( const QString & aSubDir )
{
	QStringList tList=configuration.getSources("SUBDIRS");
	tList.append(aSubDir);
	configuration.setSources("SUBDIRS",tList);
	new TrollProjectItem(this, aSubDir,path()+"/"+aSubDir);
	saveConfig();
}

//------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectItem::isWin32Scope( )
{
	return(scopeString.upper()=="WIN32");
}

QPtrList<TrollProjectItem> TrollProjectItem::getDepTargets(QPtrList<TrollProjectItem>& aCircleList)
{
	QPtrList<TrollProjectItem> tRetList;
	if(aCircleList.find(this) > -1)
	{
		return(tRetList);
	}
	aCircleList.append(this);
	
	QStringList tDeps=configuration.getTargetDeps();
	QStringList::const_iterator it;
	TrollProjectItem* tItem;
	for ( it = tDeps.begin(); it != tDeps.end(); ++it )
	{
		tItem=rootProject()->findProjectByTarget((*it));
		if(tItem)
		{
			QPtrList<TrollProjectItem> tChildDeps=tItem->getDepTargets(aCircleList);
			QPtrListIterator <TrollProjectItem> it3(tChildDeps);
			it3.toLast();
			for ( ; (*it3); --it3 )
			{
				tRetList.prepend((*it3));
			}
			tRetList.prepend(tItem);
			
		}
	}
/*	QPtrList<TrollProjectItem> tList=tRetList;
			
	QPtrListIterator <TrollProjectItem> it2(tList);
	it2.toLast();// last item will become the most signifikant
	for ( ; (*it2); --it2 )
	{
		QPtrList<TrollProjectItem> tChildDeps=tItem->getDepTargets(aCircleList);
		QPtrListIterator <TrollProjectItem> it3(tChildDeps);
		for ( ; (*it3); ++it3 )
		{
			tRetList.append((*it2));
		}
	}*/
	
	
	return(tRetList);
}

//------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectItem * TrollProjectItem::rootProject( )
{
	return((TrollProjectItem*)this->listView()->firstChild());
}

//------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectItem * TrollProjectItem::findProjectByTarget( const QString & aTarget )
{

	QFileInfo fi(aTarget);
	QString tCleanTarget=QDir::cleanDirPath(aTarget);
	int tDownDirs=tCleanTarget.contains("../");
	QString downs;
	if(tDownDirs>0)for(int i=0;i<tDownDirs;++i)downs+="../";
	
	if(this->getTargetName()==fi.fileName())
	{
		if(QDir::cleanDirPath(downs+this->getRelativPath()+"/"+this->getTargetName()) ==  QDir::cleanDirPath(aTarget))
			return(this);

	}
	//find in subprojects
	TrollProjectItem *tRetItem=NULL;
	TrollProjectItem *tItem=(TrollProjectItem*)this->firstChild();
	while(tItem && !tRetItem)
	{
		tRetItem=tItem->findProjectByTarget(aTarget );
		tItem=(TrollProjectItem*)tItem->nextSibling();
	}
	return(tRetItem);
}

//------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::getTargetName( )
{
	if(this->configuration.m_template == QTMP_APPLICATION)
	{
		if (configuration.m_target.isEmpty())
		{
			if(isWin32Scope())return configuration.m_subdirName.section('/',-1,-1,QString::SectionSkipEmpty)+".exe";
			else return configuration.m_subdirName.section('/',-1,-1,QString::SectionSkipEmpty);
		}
		else
			return configuration.m_target;
	}
	if(this->configuration.m_template == QTMP_LIBRARY)
	{
		if(configuration.m_requirements & QD_SHARED)
		{
			if(configuration.m_target!="")	return(configuration.m_target);
			else
			{
				if(isWin32Scope())return(this->subdir+".dll");
				else return(this->subdir);
			}
		}
		else if(configuration.m_requirements & QD_STATIC)
		{

			QString libString;
			if(configuration.m_target!="")
			{
				if(isWin32Scope())libString = this->configuration.m_target+".lib";
				else libString = "lib"+this->configuration.m_target+".a";

			}
			else
			{
				if(isWin32Scope())libString = this->configuration.m_subdirName.section('/',-1,-1,QString::SectionSkipEmpty)+".lib";
				else libString = "lib"+this->configuration.m_subdirName.section('/',-1,-1,QString::SectionSkipEmpty)+".a";

			}
			return(libString);
		}

	}
	return("");
}

//------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::getTargetBaseName( )
{
		if (configuration.m_target.isEmpty())
		{
			return configuration.m_subdirName.section('/',-1,-1,QString::SectionSkipEmpty);
		}
		else
			return configuration.m_target;
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::showInstallObjects( KListView * aListView )
{

	// Create list view items
	QDict<InstallDataObject> *tInstalls=configuration.getInstalls(false);
	QDictIterator <InstallDataObject> it(*tInstalls);
	GroupItem * tRootItem = new GroupItem(aListView,GroupItem::InstallRoot, "INSTALLS",scopeString);
	for(;it.current();++it)
	{
		GroupItem* tInstItem = new GroupItem(tRootItem,GroupItem::InstallObject,it.currentKey(), it.current());
		QStringList::iterator it2 = (*it)->files.begin();
		for (;it2!=(*it)->files.end();++it2)
		{
			new FileItem(tInstItem,(*it2),false);
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------------------------
GroupItem::GroupItem( TrollBaseItem * lv, GroupType type,const QString&text, InstallDataObject *aObj ):TrollBaseItem(Group,lv,text)
{
	myInstallObj=aObj;
	groupType=type;
}

//------------------------------------------------------------------------------------------------------------------------------------------
InstallDataObject * GroupItem::installObj( )
{
	return(myInstallObj);
}

//------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectItem::getTemplateString( )
{
	switch(configuration.m_template)
	{
		case QTMP_APPLICATION:
			return("application");
		case QTMP_LIBRARY:
			{
				if(configuration.m_requirements & QD_SHARED)
				{
					return("shared library");
				}
				if(configuration.m_requirements & QD_STATIC)
				{
					return("static library");
				}
				if(configuration.m_requirements & QD_PLUGIN)
				{
					return("shared library (plugin)");
				}
			}
			break;
		case QTMP_SUBDIRS:
			return("subdirs");
	}
	return("");
}

//------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectItem* TrollProjectItem::getScope(const QString &scopeString)
{
	QStringList baseScopeParts = QStringList::split(':',this->scopeString);
	QStringList subScopeParts = QStringList::split(':',scopeString);
	// Stop if baseItem not an ansister
	if (baseScopeParts.count() > subScopeParts.count())
		return NULL;
	uint i;
	for (i=0; i<baseScopeParts.count(); i++)
	{
		// Stop if baseItem in wrong treepart
		if (baseScopeParts[i] != subScopeParts[i])
			return NULL;
	}
	// if all scopeparts matched and the amount of parts are equal this must be it
	if (baseScopeParts.count() == subScopeParts.count())
		return this;
	// process next step of recursive function
	QString nextScopePart = subScopeParts[i];
	QPtrListIterator<TrollProjectItem> spit(this->scopes());
	for (; spit.current(); ++spit)
	{
		TrollProjectItem *spitem = spit;
		if (spitem->text(0)==nextScopePart)
		{
			return (*spit)->getScope(scopeString);
			break;
		}
	}
	return NULL;
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollBaseItem::paintCell( QPainter * p, const QColorGroup & cg, int column, int width, int alignment )
{
	if ( isBold() )
	{
		QFont font( p->font() );
		font.setBold( true );
		p->setFont( font );
	}
	QListViewItem::paintCell( p, cg, column, width, alignment );

}

//------------------------------------------------------------------------------------------------------------------------------------------
ProFileWatcher::ProFileWatcher( TrollProjectItem * parent, const QString & aProFile )
{
	myTrollProject=parent;
	myProFile=aProFile;
	myDirWatch.addFile(aProFile);
	connect(&myDirWatch,SIGNAL(dirty(const QString&)),this,SLOT(slotFileDirty(const QString&)));
	
}

//------------------------------------------------------------------------------------------------------------------------------------------
void ProFileWatcher::slotFileDirty( const QString & )
{
	int r = KMessageBox::questionYesNo(NULL, i18n("QMake Profile ")+myProFile+ i18n(" changed!\n Do you want do reload it?"));
	if (r == KMessageBox::Yes && myTrollProject )
	{
		myTrollProject->reload();
	}
	
}

//------------------------------------------------------------------------------------------------------------------------------------------
void ProFileWatcher::startScan( )
{
	myDirWatch.startScan(true);
}

//------------------------------------------------------------------------------------------------------------------------------------------
void ProFileWatcher::stopScan( )
{
	myDirWatch.stopScan();
}

//------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectItem::reload( )
{
	kdDebug(9024) << "reload qmake project" << endl;
	while(this->childCount()>0) this->removeItem(this->firstChild());
	QString proname = myPath + "/" + pro_file;

	m_FileBuffer.bufferFile(proname);
	m_FileBuffer.handleScopes();
	configuration.parse(&m_FileBuffer);
	parseScopes(&m_FileBuffer);
	configuration.m_subdirName=this->text(0);
	init();
	this->listView()->repaint();
}
