/***************************************************************************
 *   Copyright (C) 2003 by Thomas Hasart
 *   thasart@gmx.de
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version
 *
 ***************************************************************************/



#ifndef TROLLPROJECTITEM_H
#define TROLLPROJECTITEM_H
#include <kdeversion.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qlistview.h>
#include <qptrlist.h>
#include "filebuffer.h"
#include "trollprojectconfig.h"
#include <kdirwatch.h>
class TrollProjectPart;
class KListView;
class TrollProjectItem;
class GroupItem;
class FileItem;
class FileBuffer;

	  
/**
 * Base class for all items appearing in ProjectOverview and ProjectDetails.
 */
//-------------------------------------------------------------------------------------------------------------------------------------------
class TrollBaseItem : public QListViewItem
{
	public:
		enum Type { Subproject, Group, File };

		TrollBaseItem(Type type, QListView *parent, const QString &text);
		TrollBaseItem(Type type, TrollBaseItem *parent, const QString &text);

		QString scopeString;
		Type type()			{ return typ; }
		void paintCell(QPainter *p, const QColorGroup &cg,int column, int width, int alignment);
		void setBold(bool b)		{ myBold = b; }
		bool isBold() const		{ return myBold; }

	private:
		Type typ;
		bool myBold;
		void init();

};

//-------------------------------------------------------------------------------------------------------------------------------------------
class GroupItem : public TrollBaseItem
{
	public:
		enum GroupType {NoType, Sources, Headers, Forms,Distfiles,Images,Lexsources,Yaccsources,Translations,IDLs, InstallRoot, InstallObject};

		static GroupType groupTypeForExtension(const QString &ext);

		GroupItem(QListView *lv, GroupType type, const QString &text,const QString &scopeString);
		GroupItem(TrollBaseItem *lv, GroupType type, const QString& text, InstallDataObject *aObj );
		InstallDataObject* installObj();
		
		// qmake INSTALLS support
		InstallDataObject	*myInstallObj;
		QString             install_objectname;
		QString             install_path;
		QPtrList<GroupItem> installs;
		QPtrList<FileItem>  files;

		QStringList         str_files;
		QStringList         str_files_exclude;
		// end qmake INSTALLS support
		GroupType           groupType;
		TrollProjectItem*     owner;
};

class ProFileWatcher: public QObject
{
	Q_OBJECT

	public:
		ProFileWatcher(TrollProjectItem* parent,const QString & aProFile);
		void startScan();
		void stopScan();
	private slots:
		void slotFileDirty(const QString&);
	private:
		KDirWatch myDirWatch;
		TrollProjectItem* myTrollProject;
		QString myProFile;
};
/**
 * Stores the content of one .pro file
 */
//-------------------------------------------------------------------------------------------------------------------------------------------
class TrollProjectItem : public TrollBaseItem
{
	public:
		TrollProjectItem(QListView *parent, const QString &text,const QString &aPath);
		TrollProjectItem(TrollProjectItem *parent, const QString &text, const QString &aPath);

		//for scopes
		TrollProjectItem(TrollProjectItem *parent, const QString &text, const QString &aPath,FileBuffer *aScopeBuffer);
		~TrollProjectItem();

		QPtrList<GroupItem> groups;
		QPtrList<TrollProjectItem> scopes();

		// Containers of included an excluded files


		TrollProjectConfig configuration;

		FileBuffer m_FileBuffer;
		FileBuffer *m_RootBuffer;
		
		QString getRelativPath();
		QString getDownDirs();
		QString getLibAddPath(const QString& downDirs);
		QString getLibAddObject(const QString& downDirs);
		QString getSharedLibAddObject(const QString& downDirs);
		QString getApplicationObject(const QString& downDirs);
		QString getIncAddPath(const QString& downDirs);
		bool buildable();
		bool runable();
		bool isSubDir();
		bool canHaveSources();
		bool isRoot();
		bool isWin32Scope();

		void updateProjectFile();
		void updateIcon();
		void showSources(KListView *aListView);
		void showSources( KListView * aListView , QString aKey, GroupItem::GroupType aType);
		QPtrList <TrollProjectItem> getAllProjects();
		void getAllSubProjects(TrollBaseItem *aItem, QPtrList <TrollProjectItem> *aList);
		bool isScope();
		QString path();
		TrollProjectItem * findFile(QString aAbsFileName);
		QStringList getAllFiles();
		void saveConfig();
		void readConfig( );
		QStringList addFiles();
		QString removeFile(const QString& aFileName);
		QStringList addFiles(const QStringList & aList);
		void addFile(const QString & aFileName);
		void removeProject();
		QString pro_file;

		void createScope(const QString& aName);
		void saveScopes(FileBuffer *aBuffer );
		void parseScopes(FileBuffer *aBuffer);

		TrollProjectItem* getProject();
		bool hasFile(const QString & aFileName);
		void addFileToExcluded(const QString& aFileName);

		QString groupStringByExtension(const QString &ext);
		void addSubDir(const QString & aSubDirName);
		
		//QPtrList<TrollProjectItem> getDepTargets( );
		QPtrList<TrollProjectItem> getDepTargets(QPtrList<TrollProjectItem>& aCircleList);
		TrollProjectItem* rootProject();
		TrollProjectItem* findProjectByTarget(const QString& aTarget);
		QString getTargetName();
		
		void showInstallObjects(KListView *aListView);
		QString getTemplateString();
		TrollProjectItem* getScope(const QString &scopeString);
		QString getHeader();
		QString getTargetBaseName( );
		void removeScope();
		void reload();
	private:
		void removeFile(const QString& aKey,const QString& aFileName);
		void init();
		bool myIsScope;
		QString myPath;
		
		QString subdir;
		QString relpath;
		ProFileWatcher* myProWatch;
		
};

//-------------------------------------------------------------------------------------------------------------------------------------------
class ProjectConfigurationDlg;

//-------------------------------------------------------------------------------------------------------------------------------------------
class InsideCheckListItem:public QCheckListItem
{
	public:
		InsideCheckListItem(QListView *parent,TrollProjectItem *item, ProjectConfigurationDlg *config):
//				QCheckListItem(parent, item->text(0), QCheckListItem::CheckBox)
//				QCheckListItem(parent, item->relpath.right(item->relpath.length()-1), QCheckListItem::CheckBox)
				QCheckListItem(parent, item->getRelativPath().right(item->getRelativPath().length()-1), QCheckListItem::CheckBox)
		{
			prjItem=item;
			m_config = config;
		}
		InsideCheckListItem(QListView *parent,QListViewItem *after,TrollProjectItem *item, ProjectConfigurationDlg *config):
				QCheckListItem(parent,
#if KDE_VERSION > 305
				               after,
#endif
//				               item->text(0),QCheckListItem::CheckBox)
//				               item->relpath.right(item->relpath.length()-1),QCheckListItem::CheckBox)
				               item->getRelativPath().right(item->getRelativPath().length()-1),QCheckListItem::CheckBox)
		{
			prjItem=item;
			m_config = config;
		}
		TrollProjectItem *prjItem;
		ProjectConfigurationDlg *m_config;

};





// Not sure if this complexity is really necessary...
//-------------------------------------------------------------------------------------------------------------------------------------------
class FileItem : public TrollBaseItem
{
	public:
		FileItem(TrollBaseItem *parent, const QString &text,bool exclude=false);
		//FileItem(QListView *lv, const QString &text,bool exclude=false);

		QString uiFileLink;
		QString name;
		bool excluded;
};

#endif
