/***************************************************************************
 *   Copyright (C) 2003 - 2004 by Thomas Hasart
 *   thasart@gmx.de
 *
 *   Copyright (C) 2001 by Bernd Gehrmann
 *   bernd@kdevelop.org
 *
 *   Copyright (C) 2000-2001 by Trolltech AS
 *   info@trolltech.com
 *
 *   Copyright (C) 2002 by Jakob Simon-Gaarde
 *   jakob@jsg.dk
 *
 *   Copyright (C) 2002-2003 by Alexander Dymo
 *   cloudtemple@mksat.net
 *
 *   Part of this file is taken from Qt Designer
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version
 *
 ***************************************************************************/

#include "trollprojectwidget.h"

#include <config.h>

#include <qfile.h>
#include <qfileinfo.h>
#include <qheader.h>
#include <qsplitter.h>
#include <qptrstack.h>
#include <qtextstream.h>
#include <qcombobox.h>
#include <qprocess.h>
#include <qtimer.h>
#include <qdir.h>
#include <qregexp.h>
#include <qinputdialog.h>
#include <qwhatsthis.h>
#include <kfiledialog.h>
#include <qtooltip.h>
#include <kdebug.h>
#include <klistview.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <kregexp.h>
#include <kurl.h>
#include <qmessagebox.h>
#include <iostream>
#include <kparts/part.h>
#include <kaction.h>
#include <kprocess.h>
#include <klineeditdlg.h>
#include <kdeversion.h>
#include <kurlrequesterdlg.h>
#include <kurlrequester.h>
#include <kio/netaccess.h>
#include <kinputdialog.h>
#include <ktoolbar.h>
#include <tpconfigdialog.h>
#include "kdevcore.h"
#include "kdevpartcontroller.h"
#include "kdevmainwindow.h"
#include "trollprojectpart.h"
#include "kdevappfrontend.h"
#include "kdevmakefrontend.h"
#include "kdevlanguagesupport.h"
#include "kdevcreatefile.h"
#include "subclassesdlg.h"
#include "addfilesdialog.h"
#include "urlutil.h"
#include "pathutil.h"

#define VALUES_PER_ROW  1
#define MAKE_COMMAND "make"

/*
 * Class TrollBaseItem
 */


//-------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectWidget::TrollProjectWidget(TrollProjectPart *part)
		: QVBox(0, "troll project widget")
{
	m_part = part;
	myConfigDialog=NULL;
	myMainTarget=NULL;
	connect(m_part->partController(),SIGNAL(fileDirty(const QString&)),this,SLOT(slotFileDirty(const QString&)));
	connect(m_part->partController(),SIGNAL(fileDirty(const KURL&)),this,SLOT(slotFileDirty(const KURL&)));
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::init(TrollProjectPart *part)
{
	m_part = part;
	QSplitter *splitter = new QSplitter(Vertical, this);
	myMainTarget=NULL;

	//////////////////
	// PROJECT VIEW //
	//////////////////

	overviewContainer = new QVBox(splitter,"Projects");
	overviewContainer->setMargin ( 2 );
	overviewContainer->setSpacing ( 2 );
	// spacer
	//projectTools->setStretchFactor(spacer, 1);
	// Project configuration
	KToolBar *tBar=new KToolBar(overviewContainer);
	tBar->setIconSize(16);
	tBar->setFullSize(true);

	m_part->actionCollection()->action("config_add_subproject")->plug(tBar);
	m_part->actionCollection()->action("config_create_scope")->plug(tBar);
	m_part->actionCollection()->action("build_compilefile")->plug(tBar);
	m_part->actionCollection()->action("build_build_project")->plug(tBar);
	m_part->actionCollection()->action("build_rebuild_project")->plug(tBar);
	m_part->actionCollection()->action("build_execute_project")->plug(tBar);



	QWidget *spacer = new QWidget(tBar);
	tBar->insertWidget(99,1,spacer);
	tBar->setStretchableWidget (spacer);
	m_part->actionCollection()->action("config_subproject")->plug(tBar);

	//tBar->alignItemRight (id,true);




	// Project tree
	overview = new KListView(overviewContainer, "project overview widget");
	overview->setResizeMode(QListView::LastColumn);
	overview->setSorting(-1);
	overview->header()->hide();
	overview->addColumn(QString::null);

	// Project tree connections
	connect( overview, SIGNAL(selectionChanged(QListViewItem*)),
	         this, SLOT(slotOverviewSelectionChanged(QListViewItem*)) );
	connect( overview, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint&)),
	         this, SLOT(slotOverviewContextMenu(KListView*, QListViewItem*, const QPoint&)) );


	/////////////////
	// DETAIL VIEW //
	/////////////////

	// Details tree
	detailContainer = new QVBox(splitter,"Details");
	detailContainer->setMargin ( 2 );
	detailContainer->setSpacing ( 2 );

	tBar=new KToolBar(detailContainer);
	tBar->setIconSize(16);
	tBar->setFullSize(true);

	m_part->actionCollection()->action("source_new_file")->plug(tBar);
	m_part->actionCollection()->action("source_add_files")->plug(tBar);
	m_part->actionCollection()->action("source_remove_file")->plug(tBar);
	m_part->actionCollection()->action("build_build_target")->plug(tBar);
	m_part->actionCollection()->action("build_rebuild_target")->plug(tBar);
	m_part->actionCollection()->action("build_execute_target")->plug(tBar);

	spacer = new QWidget(tBar);
	tBar->insertWidget(99,1,spacer);
	tBar->setStretchableWidget (spacer);
	m_part->actionCollection()->action("source_config_file")->plug(tBar);


	// detail tree
	details = new KListView(detailContainer, "details widget");
	details->setRootIsDecorated(true);
	details->setResizeMode(QListView::LastColumn);
	details->setSorting(-1);
	details->header()->hide();
	details->addColumn(QString::null);

	// Detail tree connections
	connect( details, SIGNAL(selectionChanged(QListViewItem*)),this, SLOT(slotDetailsSelectionChanged(QListViewItem*)) );
	connect( details, SIGNAL(executed(QListViewItem*)),this, SLOT(slotDetailsExecuted(QListViewItem*)) );
	connect( details, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint&)),this, SLOT(slotDetailsContextMenu(KListView*, QListViewItem*, const QPoint&)) );


	m_shownSubproject = 0;
	m_rootSubproject = 0;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectWidget::~TrollProjectWidget()
{}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::openProject(const QString &dirName)
{
	m_subclasslist = DomUtil::readPairListEntry(*m_part->projectDom(),"/kdevtrollproject/subclassing" , "subclass","sourcefile", "uifile");
	TrollProjectItem *item = new TrollProjectItem(overview, "/",dirName);
	m_rootSubproject=item;
	item->setOpen(true);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::closeProject()
{
	m_rootSubproject = 0;
	overview->clear();
	details->clear();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QStringList TrollProjectWidget::allSubprojects()
{
	int prefixlen = projectDirectory().length()+1;
	QStringList res;

	QListViewItemIterator it(overview);
	for (; it.current(); ++it)
	{
		if (it.current() == overview->firstChild())
			continue;
		QString path = static_cast<TrollProjectItem*>(it.current())->path();
		res.append(path.mid(prefixlen));
	}

	return res;

}


//-------------------------------------------------------------------------------------------------------------------------------------------
QStringList TrollProjectWidget::allFiles()
{
	return(m_rootSubproject->getAllFiles());
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectWidget::projectDirectory()
{
	if (!overview->firstChild())
		return QString::null; //confused
	//return("/root/prj/minder3");

	return static_cast<TrollProjectItem*>(overview->firstChild())->path();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectWidget::subprojectDirectory()
{
	if (!m_shownSubproject)
		return QString::null;

	return m_shownSubproject->path();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::setupContext()
{
	TrollProjectItem *tPrjItem=static_cast<TrollProjectItem*>(overview->currentItem());
	if (!tPrjItem)return;

	// Setup project view
	m_part->actionCollection()->action("build_build_target")->setEnabled(tPrjItem->buildable());
	m_part->actionCollection()->action("build_rebuild_target")->setEnabled(tPrjItem->buildable());
	m_part->actionCollection()->action("build_execute_target")->setEnabled(tPrjItem->runable());
	m_part->actionCollection()->action("config_add_subproject")->setEnabled(tPrjItem->isSubDir());
	m_part->actionCollection()->action("config_remove_subproject")->setEnabled(tPrjItem->isRoot());
	m_part->actionCollection()->action("config_create_scope")->setEnabled(!tPrjItem->isSubDir());


	// Setup source view

	details->setEnabled(tPrjItem->canHaveSources());
	m_part->actionCollection()->action("source_new_file")->setEnabled(tPrjItem->canHaveSources());
	m_part->actionCollection()->action("source_add_files")->setEnabled(tPrjItem->canHaveSources());

	TrollBaseItem *tSrcItem=static_cast<TrollBaseItem*>(details->currentItem());
	if (!tSrcItem)
	{
		m_part->actionCollection()->action("source_config_file")->setEnabled(false);
		m_part->actionCollection()->action("source_remove_file")->setEnabled(false);
	}
	else
	{
		m_part->actionCollection()->action("source_config_file")->setEnabled(tSrcItem->type()==TrollBaseItem::File);
		m_part->actionCollection()->action("source_remove_file")->setEnabled(tSrcItem->type()==TrollBaseItem::File);
	}
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotOverviewSelectionChanged(QListViewItem *item)
{
	if (!item)
		return;
	m_shownSubproject = static_cast<TrollProjectItem*>(item);
	m_shownSubproject->showSources(details);
	setupContext();

	if(myConfigDialog)
	{
		myConfigDialog->setProject(m_shownSubproject);
		myConfigDialog->show();

	}
	DomUtil::writeEntry(*m_part->projectDom(), "/kdevtrollproject/general/activedir",m_shownSubproject->getRelativPath());

}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectWidget::getCurrentTarget()
{
	if (!m_shownSubproject)
		return "";
	if (m_shownSubproject->configuration.m_destdir.isEmpty() ||
	        m_shownSubproject->configuration.m_destdir[ m_shownSubproject->configuration.m_destdir.length()-1 ] == '/' )
		return m_shownSubproject->configuration.m_destdir+m_shownSubproject->configuration.m_target;
	else
		return m_shownSubproject->configuration.m_destdir+'/'+m_shownSubproject->configuration.m_target;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectWidget::getCurrentDestDir()
{
	return "";
	/*	if (!m_shownSubproject)
			return "";
		return m_shownSubproject->configuration.m_destdir;*/
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotDetailsExecuted(QListViewItem *item)
{
	setupContext();
	if (!item)
		return;

	// We assume here that ALL items in both list views
	// are TrollBaseItem's
	TrollBaseItem *pvitem = static_cast<TrollBaseItem*>(item);
	if (pvitem->type() != TrollBaseItem::File)
		return;

	QString dirName = m_shownSubproject->path();
	FileItem *fitem = static_cast<FileItem*>(pvitem);

	bool isUiFile = QFileInfo(fitem->name).extension() == "ui";
	if( m_part->isTMakeProject() && isUiFile )
	{
		// start designer in your PATH
		KShellProcess proc;
		proc << "designer" << (dirName + "/" + QString(fitem->name));
		proc.start( KProcess::DontCare, KProcess::NoCommunication );

	}
	else
		m_part->partController()->editDocument(KURL(dirName + "/" + QString(fitem->name)));

	m_part->mainWindow()->lowerView(this);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotConfigureProject()
{
	if(!myConfigDialog)
	{
		myConfigDialog=new TPConfigDialog(m_shownSubproject, this, "ConfigDialog",false);
		myConfigDialog->resize(600,300);
		connect(myConfigDialog,SIGNAL(finished()),this,SLOT(slotConfigFinished()));
		connect(myConfigDialog,SIGNAL(applyClicked()),this,SLOT(slotApplyClicked()));
		connect(myConfigDialog,SIGNAL(okClicked()),this,SLOT(slotOkClicked()));
		connect(myConfigDialog,SIGNAL(finished()),this,SLOT(slotCancelClicked()));
	}
	myConfigDialog->show();
}

void TrollProjectWidget::slotApplyClicked()
{
	m_shownSubproject->saveConfig();
	m_shownSubproject->updateIcon();
	setupContext();
}
void TrollProjectWidget::slotOkClicked()
{
	m_shownSubproject->saveConfig();
	m_shownSubproject->updateIcon();
	setupContext();
}
void TrollProjectWidget::slotCancelClicked()
{
	m_shownSubproject->readConfig();
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotConfigFinished()
{
	delete(myConfigDialog);
	myConfigDialog=NULL;
}





//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotCreateScope(TrollProjectItem *tPrjItem)
{
	if (tPrjItem==0 && m_shownSubproject==0)
		return;
	else
		tPrjItem = m_shownSubproject;
	bool ok = FALSE;
	QString scopename = KInputDialog::getText(
	                        i18n( "Create Scope" ),
	                        i18n( "Please enter a name for the new scope:" ),
	                        QString::null, &ok, this );
	if ( ok && !scopename.isEmpty() )
	{
		tPrjItem->createScope(scopename);
	}
	else
		return;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotAddSubdir(TrollProjectItem *tPrjItem)
{
	if (tPrjItem==0 && m_shownSubproject==0)
		return;
	else
		tPrjItem = m_shownSubproject;
	QString relpath = tPrjItem->path().mid(projectDirectory().length());

	KURLRequesterDlg dialog(i18n( "Add Subdirectory" ), i18n( "Please enter a name for the subdirectory: " ), this, 0);
	dialog.urlRequester()->setMode(KFile::Directory);
	dialog.urlRequester()->setURL(QString::null);

	if ( dialog.exec() == QDialog::Accepted && !dialog.urlRequester()->url().isEmpty() )
	{
		QString subdirname;
		if ( !QDir::isRelativePath(dialog.urlRequester()->url()) )
			subdirname = getRelativePath( m_shownSubproject->path(), dialog.urlRequester()->url() );
		else
			subdirname = dialog.urlRequester()->url();

		QDir dir(projectDirectory()+relpath);
		if (!dir.exists(subdirname))
		{
			if (!dir.mkdir(subdirname))
			{
				KMessageBox::error(this,i18n("Failed to create subdirectory. "
				                             "Do you have write permission "
				                             "in the project folder?" ));
				return;
			}
		}
		tPrjItem->addSubDir(subdirname);
		/*tPrjItem->subdirs.append(subdirname);
		//updateProjectFile(tPrjItem);
		TrollProjectItem *newitem = new TrollProjectItem(tPrjItem, subdirname,"");
		newitem->subdir = subdirname;
		newitem->m_RootBuffer = &(newitem->m_FileBuffer);
		newitem->path = tPrjItem->path + "/" + subdirname;
		newitem->relpath = newitem->path;
		newitem->relpath.remove(0,projectDirectory().length());

		parse(newitem);*/
	}
	else
		return;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotRemoveSubproject()
{
	TrollProjectItem *tPrjItem=static_cast<TrollProjectItem*>(overview->currentItem());
	if (!tPrjItem)return;

	tPrjItem->removeProject();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotOverviewContextMenu(KListView *, QListViewItem *item, const QPoint &p)
{
	if (!item)
		return;

	//TrollProjectItem *tPrjItem = static_cast<TrollProjectItem*>(item);

	KPopupMenu popup( this);
	popup.insertTitle(i18n("Subproject %1").arg(item->text(0)));

	if (1)//!tPrjItem->isScope)
	{

		m_part->actionCollection()->action("config_subproject")->plug(&popup);
		popup.insertSeparator();
		m_part->actionCollection()->action("build_build_target")->plug(&popup);
		m_part->actionCollection()->action("build_run_qmake")->plug(&popup);
		popup.insertSeparator();
		m_part->actionCollection()->action("build_rebuild_target")->plug(&popup);
		m_part->actionCollection()->action("build_relink_target")->plug(&popup);
		popup.insertSeparator();

		m_part->actionCollection()->action("build_create_menu")->plug(&popup);
		popup.insertSeparator();
		m_part->actionCollection()->action("build_buidmode_menu")->plug(&popup);
		popup.insertSeparator();
		m_part->actionCollection()->action("build_make_menu")->plug(&popup);
		m_part->actionCollection()->action("build_make_menu_targetonly")->plug(&popup);
		m_part->actionCollection()->action("build_make_menu_project")->plug(&popup);
		m_part->actionCollection()->action("set_main_target")->plug(&popup);
		m_part->actionCollection()->action("build_extra_menu")->plug(&popup);


	}
	else
	{
		m_part->actionCollection()->action("config_create_scope")->plug(&popup);
		m_part->actionCollection()->action("config_remove_scope")->plug(&popup);
	}
	popup.exec(p);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotDetailsContextMenu(KListView *, QListViewItem *item, const QPoint &p)
{
	if (!item)
		return;

	KPopupMenu popup( this);
	TrollBaseItem *tBaseItem = static_cast<TrollBaseItem*>(item);

	if(tBaseItem->type()==TrollBaseItem::File)
	{
		FileItem *tFileItem=static_cast<FileItem*>(item);
		m_part->actionCollection()->action("source_remove_file")->plug(&popup);
		m_part->actionCollection()->action("source_config_file")->plug(&popup);

		QString tFileName=tFileItem->text(0);
		QFileInfo fi(tFileName);
		if(fi.extension()=="ui")
		{
			popup.insertSeparator();
			m_part->actionCollection()->action("file_subclass_ui")->plug(&popup);
			m_part->actionCollection()->action("file_open_ui_header")->plug(&popup);
		}
		/* FIXME
		m_part->actionCollection()->action("file_subclass_ui_edit")->plug(&popup);
		*/

		FileContext context( m_shownSubproject->path() + "/" + tFileItem->text(0), false );
		m_part->core()->fillContextMenu( &popup, &context );

	}
	else if(tBaseItem->type()==TrollBaseItem::Group)
	{
		GroupItem *tGroupItem=(GroupItem*)tBaseItem;
		if(tGroupItem->groupType==GroupItem::InstallRoot)
		{
			m_part->actionCollection()->action("add_install_root")->plug(&popup);
		}
		else if(tGroupItem->groupType==GroupItem::InstallObject)
		{
			m_part->actionCollection()->action("set_install_path")->plug(&popup);
			m_part->actionCollection()->action("add_install_files")->plug(&popup);
			popup.insertSeparator();
			m_part->actionCollection()->action("remove_install_root")->plug(&popup);
		}
		else
		{
			m_part->actionCollection()->action("source_new_file")->plug(&popup);
			m_part->actionCollection()->action("source_add_files")->plug(&popup);
		}
	}

	popup.exec(p);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotConfigureFile()
{
		QListViewItem *selectedItem = details->currentItem();
		if (!selectedItem)
			return;
		TrollBaseItem *pvitem = static_cast<TrollBaseItem*>(selectedItem);
		// Check that it is a file (just in case)
		if (pvitem->type() != TrollBaseItem::File)
			return;
		FileItem *fitem = static_cast<FileItem*>(pvitem);
	 
		GroupItem *gitem = static_cast<GroupItem*>(fitem->parent());
		if (!gitem)
			return;
		QStringList dirtyScopes;
		FilePropertyDlg *propdlg = new FilePropertyDlg(m_shownSubproject,gitem->groupType,fitem);
		TrollProjectItem *scope;
		if(propdlg->exec()==QDialog::Accepted)
			m_shownSubproject->saveConfig();
		
	 
		for (uint i=0; i<dirtyScopes.count();i++)
		{
			//scope =m_shownSubproject->getScope(dirtyScopes[i]);
			//scope = getScope(m_shownSubproject,dirtyScopes[i]);
			if (gitem->groupType == GroupItem::InstallObject)
			{
/*				GroupItem* instroot = scope->getInstallRoot(scope);
				GroupItem* instobj = getInstallObject(scope,gitem->install_objectname);
				if (!instobj)
				{
					GroupItem* institem = createGroupItem(GroupItem::InstallObject, gitem->install_objectname ,scope->scopeString);
					institem->owner = scope;
					institem->install_objectname = gitem->install_objectname;
					instroot->installs.append(institem);
					instobj = institem;
				}
				// Using the boolean nature of this operation I can append or remove the file from the excludelist
				if (instobj->str_files_exclude.join(":").find(fitem->name) >= 0)
				{
					instobj->str_files_exclude.remove(fitem->name);
				}
				else
				{
					instobj->str_files_exclude.append(fitem->name);
				}
*/			}
			if (scope);
				//updateProjectFile(scope);
		}
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotDetailsSelectionChanged(QListViewItem *)//item)
{

	setupContext();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::emitAddedFile(const QString &fileName)
{
	QStringList fileList;
	fileList.append ( fileName );
	emit m_part->addedFilesToProject(fileList);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::emitRemovedFile(const QString &fileName)
{
	QStringList fileList;
	fileList.append ( fileName );
	emit m_part->removedFilesFromProject(fileList);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectWidget::getUiFileLink(const QString &relpath, const QString& filename)
{
	DomUtil::PairList::iterator it;
	for (it=m_subclasslist.begin();it != m_subclasslist.end(); ++it)
	{
		if ((*it).first==relpath+filename)
			return (*it).second;
	}
	return "";
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotBuildFile()
{
	KParts::ReadWritePart *part = dynamic_cast<KParts::ReadWritePart*>(m_part->partController()->activePart());
	if (!part || !part->url().isLocalFile())
		return;
	m_part->partController()->saveAllFiles();
	QString fileName = part->url().path();
	QFileInfo fi(fileName);
	QString sourceDir = fi.dirPath();
	QString baseName = fi.baseName(true);

	QString projectDir = projectDirectory();
	if (!sourceDir.startsWith(projectDir))
	{
		KMessageBox::sorry(this, i18n("Can only compile files in directories which belong to the project."));
		return;
	}

	QString buildDir = sourceDir;
	QString target = baseName + ".o";

	m_part->mainWindow()->raiseView(m_part->makeFrontend()->widget());

	TrollProjectItem* tPrjItem =  m_rootSubproject->findFile(fi.absFilePath());

	if(tPrjItem)
	{
		if(! tPrjItem->configuration.m_objectpath.isEmpty())
		{
			target=tPrjItem->configuration.m_objectpath;
			target+="/";
			target+=baseName + ".o";
			
		}
		QString tBuildCmd = constructMakeCommandLine(tPrjItem->configuration.m_makefile);
		QString dircmd = "cd " + tPrjItem->path() + " && " ;
		m_part->queueCmd(tPrjItem->path(), dircmd + tBuildCmd + " " + target);
	}
	else
	{
		/*QString tBuildCmd = constructMakeCommandLine(tPrjItem->configuration.m_makefile);
		QString dircmd = "cd " + sourceDir + " && " ;
		kdDebug(9020) << "builddir " <<sourceDir<< ", cmd " << dircmd + tBuildCmd + " " + target << endl;
		m_part->queueCmd(tPrjItem->path(), dircmd + tBuildCmd + " " + target);*/
	}

	m_part->mainWindow()->lowerView(this);
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotExecuteTarget()
{

	if (m_part->appFrontend()->isRunning())
	{
		if (KMessageBox::questionYesNo(this, i18n("Your application is currently running. Do you want to restart it?"), i18n("Application already running"), i18n("&Restart application"), i18n("Do &Nothing")) == KMessageBox::No)
			return;
		connect(m_part->appFrontend(), SIGNAL(processExited()), this,SLOT(slotExecuteTarget2()));
		m_part->appFrontend()->stopApplication();
		return;
	}

	slotExecuteTarget2();
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotExecuteTarget2()
{
	disconnect(m_part->appFrontend(), SIGNAL(processExited()),this,SLOT(slotExecuteTarget2()));
	if( DomUtil::readBoolEntry(*m_part->projectDom(), "/kdevtrollproject/run/autocompile", true) && m_part->isDirty() )
	{
		connect(m_part->makeFrontend(), SIGNAL(commandFinished(const QString &)), this,SLOT(slotExecuteTarget3(const QString &)));
		connect(m_part->makeFrontend(), SIGNAL(commandFailed(const QString &)), this,SLOT(slotAutoCompileTargetFailed(const QString &)));
		slotMakeCommandProject();
		return;
	}
	slotExecuteTarget3("");
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotAutoCompileTargetFailed(const QString&aCmd)
{
		m_part->appFrontend()->insertStderrLine(i18n( "autocompile target before execute failed"));
		m_part->appFrontend()->insertStderrLine(i18n( "cmd:> "+aCmd));
		disconnect(m_part->makeFrontend(), SIGNAL(commandFinished(const QString &)), this,SLOT(slotExecuteTarget3(const QString &)));
		disconnect(m_part->makeFrontend(), SIGNAL(commandFailed(const QString &)), this,SLOT(slotAutoCompileTargetFailed(const QString &)));
}
	
//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotExecuteTarget3(const QString &aCmd)
{
	if(!(aCmd.isEmpty() || aCmd==myMakeCommand))
	{
		return;
	}
	disconnect(m_part->makeFrontend(), SIGNAL(commandFinished(const QString &)), this,SLOT(slotExecuteTarget3(const QString &)));
	disconnect(m_part->makeFrontend(), SIGNAL(commandFailed(const QString &)), this,SLOT(slotAutoCompileTargetFailed(const QString &)));
	//m_part->slotExecute();

	// no subproject selected
	if (!m_shownSubproject)
		return;

	// can't build from scope
	if (m_shownSubproject->isScope())
		return;


	// Only run application projects

	QString dircmd = "cd "+subprojectDirectory() + "/" + getCurrentDestDir() + " && ";
	QString program = "./" + getCurrentOutputFilename();

	// Build environment variables to prepend to the executable path
	QString runEnvVars = QString::null;
	DomUtil::PairList list =
	    DomUtil::readPairListEntry( *(m_part->projectDom()), "/kdevtrollproject/run/envvars", "envvar", "name", "value" );

	DomUtil::PairList::ConstIterator it;
	for (it = list.begin(); it != list.end(); ++it)
	{
		const DomUtil::Pair &pair = (*it);
		if ( (!pair.first.isEmpty()) && (!pair.second.isEmpty()) )
			runEnvVars += pair.first + "=" + pair.second + " ";
	}
	program.prepend(runEnvVars);

	program.append(" " + DomUtil::readEntry( *(m_part->projectDom()), "/kdevtrollproject/run/programargs" ) + " ");
	//  std::cerr<<dircmd + "./"+program<<std::endl;
	//  m_part->execute(dircmd + "./"+program);
	//  m_part->appFrontend()->startAppCommand(dircmd +"./"+program,true);

	bool inTerminal = DomUtil::readBoolEntry(*m_part->projectDom(), "/kdevtrollproject/run/terminal");
	m_part->mainWindow()->raiseView(m_part->appFrontend()->widget());
	m_part->appFrontend()->startAppCommand(subprojectDirectory() + "/" + getCurrentDestDir(), program, inTerminal );

}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotExecuteProject()
{

	if (m_part->appFrontend()->isRunning())
	{
		if (KMessageBox::questionYesNo(this, i18n("Your application is currently running. Do you want to restart it?"), i18n("Application already running"), i18n("&Restart application"), i18n("Do &Nothing")) == KMessageBox::No)
			return;
		connect(m_part->appFrontend(), SIGNAL(processExited()), this,SLOT(slotExecuteProject2()));
		m_part->appFrontend()->stopApplication();
		return;
	}

	slotExecuteProject2();
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotExecuteProject2()
{
	disconnect(m_part->appFrontend(), SIGNAL(processExited()), this,SLOT(slotExecuteProject2()));
	if( DomUtil::readBoolEntry(*m_part->projectDom(), "/kdevtrollproject/run/autocompile", true) && m_part->isDirty() )
	{
		connect(m_part->makeFrontend(), SIGNAL(commandFinished(const QString &)), this,SLOT(slotExecuteProject3(const QString &)));
		connect(m_part->makeFrontend(), SIGNAL(commandFailed(const QString &)), this,SLOT(slotAutoCompileProjectFailed(const QString &)));
		slotMakeCommandProject();
		return;
	}
	slotExecuteProject3("");
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotAutoCompileProjectFailed(const QString&aCmd)
{
		m_part->appFrontend()->insertStderrLine(i18n( "autocompile project before execute failed"));
		m_part->appFrontend()->insertStderrLine(i18n( "cmd:> "+aCmd));
		disconnect(m_part->makeFrontend(), SIGNAL(commandFinished(const QString &)), this,SLOT(slotExecuteProject3(const QString &)));
		disconnect(m_part->makeFrontend(), SIGNAL(commandFailed(const QString &)), this,SLOT(slotAutoCompileProjectFailed(const QString &)));
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotExecuteProject3(const QString& aCmd)
{
	if(!(aCmd.isEmpty() || aCmd==myMakeCommand))
	{
		return;
	}
	disconnect(m_part->makeFrontend(), SIGNAL(commandFinished(const QString &)), this,SLOT(slotExecuteProject3(const QString &)));
	disconnect(m_part->makeFrontend(), SIGNAL(commandFailed(const QString &)), this,SLOT(slotAutoCompileProjectFailed(const QString &)));
	
	
	QString program = m_part->mainProgram();
	if (!program.startsWith("/"))
		program.prepend("./");

	if ( program.isEmpty() )
	{
		KMessageBox::sorry(this, i18n("Please specify the executable name in the "
		                              "project options dialog first.Or Select an App-Project"), i18n("No Executable Name Given"));
		return;
	}

	// Build environment variables to prepend to the executable path
	QString runEnvVars = QString::null;
	DomUtil::PairList list =
	    DomUtil::readPairListEntry( *(m_part->projectDom()), "/kdevtrollproject/run/envvars", "envvar", "name", "value" );

	DomUtil::PairList::ConstIterator it;
	for (it = list.begin(); it != list.end(); ++it)
	{
		const DomUtil::Pair &pair = (*it);
		if ( (!pair.first.isEmpty()) && (!pair.second.isEmpty()) )
			runEnvVars += pair.first + "=" + pair.second + " ";
	}
	program.prepend(runEnvVars);
	program.append(" " + DomUtil::readEntry( *(m_part->projectDom()), "/kdevtrollproject/run/programargs" ) + " ");

	QString dircmd = "cd "+this->projectDirectory() + " && " ;

	bool inTerminal = DomUtil::readBoolEntry(*(m_part->projectDom()), "/kdevtrollproject/run/terminal");
	//    m_part->appFrontend()->startAppCommand(dircmd + program, inTerminal);
	//    m_part->execute(this->projectDirectory(), "./"+program );
	m_part->mainWindow()->raiseView(m_part->appFrontend()->widget());
	m_part->appFrontend()->startAppCommand(this->projectDirectory(), program,inTerminal );
}




//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectWidget::constructMakeCommandLine(const QString makeFileName )
{
	QDomDocument &dom = *(m_part->projectDom());

	QString cmdline = DomUtil::readEntry(dom, "/kdevtrollproject/make/makebin");
	if (cmdline.isEmpty())
		cmdline = MAKE_COMMAND;
	if (!makeFileName.isEmpty())
	{
		cmdline += " -f " + makeFileName;
	}
	if (!DomUtil::readBoolEntry(dom, "/kdevtrollproject/make/abortonerror"))
		cmdline += " -k";
	int jobs = DomUtil::readIntEntry(dom, "/kdevtrollproject/make/numberofjobs");
	if (jobs != 0)
	{
		cmdline += " -j";
		cmdline += QString::number(jobs);
	}
	if (DomUtil::readBoolEntry(dom, "/kdevtrollproject/make/dontact"))
		cmdline += " -n";

	cmdline += " ";
	cmdline.prepend(m_part->makeEnvironment());

	return cmdline;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::startMakeCommand( const QString & dir, const QString & target )
{
	m_part->partController()->saveAllFiles();

	/*    QFileInfo fi;
	    QFileInfo fi2;
	    if (m_rootSubproject->configuration.m_makefile.isEmpty())
	    {
	        fi.setFile(dir + "/Makefile");
	        fi2.setFile(dir + "/makefile");
	    }
	    else
	    {
	        fi.setFile(m_rootSubproject->configuration.m_makefile);
	        fi2.setFile(dir + "/" + m_rootSubproject->configuration.m_makefile);
	    }
	    if (!fi.exists() && !fi2.exists()) {
	        int r = KMessageBox::questionYesNo(this, i18n("There is no Makefile in this directory. Run qmake first?"));
	        if (r == KMessageBox::No)
	            return;
	        m_part->startQMakeCommand(dir);
	    }
	*/
	QDomDocument &dom = *(m_part->projectDom());

	if (target=="clean")
	{
		QString cmdline = DomUtil::readEntry(dom, "/kdevtrollproject/make/makebin");
		if (cmdline.isEmpty())
			cmdline = MAKE_COMMAND;
		cmdline += " clean";
		QString dircmd = "cd " + dir + " && ";
		cmdline.prepend(m_part->makeEnvironment());
		m_part->makeFrontend()->queueCommand(dir, dircmd + cmdline);
	}

	QString cmdline = constructMakeCommandLine() + " " + target;

	QString dircmd = "cd " + dir + " && ";

	cmdline.prepend(m_part->makeEnvironment());
	m_part->makeFrontend()->queueCommand(dir, dircmd + cmdline);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::createMakefileIfMissing(const QString &dir, TrollProjectItem *item,bool ask)
{
	QFileInfo fi;
	QFileInfo fi2;
	if (item->configuration.m_makefile.isEmpty())
	{
		fi.setFile(dir + "/Makefile");
		fi2.setFile(dir + "/makefile");
	}
	else
	{
		fi.setFile(item->configuration.m_makefile);
		fi2.setFile(dir + "/" + item->configuration.m_makefile);
	}
	
	if (!fi.exists() && !fi2.exists())
	{
		if(ask)
		{
			int r = KMessageBox::questionYesNo(this, i18n("There is no Makefile in this directory. Run qmake first?"));
			if (r == KMessageBox::No)return;
		}
		m_part->startQMakeCommand(dir);
	}
}




//-------------------------------------------------------------------------------------------------------------------------------------------
//check or uncheck dependency to currently checked or unchecked library
void TrollProjectWidget::slotRemoveScope( TrollProjectItem * tPrjItem )
{
	if (!m_shownSubproject)	return;
	if(!m_shownSubproject->isScope())return;
	m_shownSubproject->removeScope();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectItem * TrollProjectWidget::findSubprojectForScope( TrollProjectItem * scope )
{
	if ((scope == 0) || (scope->parent() == 0))
		return 0;
	if (!scope->isScope())
		return scope;
	return findSubprojectForScope(dynamic_cast<TrollProjectItem *>(scope->parent()));
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotRunQMake( )
{

	TrollProjectItem *tPrjItem = static_cast<TrollProjectItem*>(overview->selectedItem());
	if(!tPrjItem)return;
	QString relpath = tPrjItem->path().mid(projectDirectory().length());
	m_part->startQMakeCommand(projectDirectory() + relpath);
	m_part->mainWindow()->lowerView(this);

}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotBuildModeDebug( )
{
	setBuildMode(NULL,false,QBM_DEBUG);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotBuildModeRelease( )
{
	setBuildMode(NULL,false,QBM_RELEASE);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotBuildModeDebugAll()
{
	setBuildMode(NULL,true,QBM_DEBUG);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotBuildModeReleaseAll( )
{
	setBuildMode(NULL,true,QBM_RELEASE);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::setBuildMode( QListViewItem * aItem, bool rekursive, QMakeBuildMode aMode )
{
	QListViewItem *tChild=NULL;
	if(!aItem)	tChild = overview->selectedItem();
	else tChild=aItem;

	//the item itself
	TrollProjectItem* tPrjItem = dynamic_cast<TrollProjectItem*>(tChild);
	TrollProjectItem* tFirstPrjItem = tPrjItem;
	if(!tPrjItem)return;
	if(tPrjItem->isScope())return;
	if(tPrjItem->configuration.m_template != QTMP_SUBDIRS)
	{
		tPrjItem->configuration.m_buildMode=aMode;
		//updateProjectConfiguration(tPrjItem);
	}

	if(rekursive)
	{
		tChild=tChild->firstChild();
		while( tChild )
		{
			setBuildMode(tChild,rekursive,aMode );
			tChild = tChild->nextSibling();
		}
	}
	tFirstPrjItem->saveConfig();

}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotAddFiles()
{
	QStringList tList=m_shownSubproject->addFiles();
	m_part->addedFilesToProject(tList);
	m_shownSubproject->showSources(details);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotRemoveFile( )
{
	TrollBaseItem *tBaseItem=static_cast<TrollBaseItem*>(details->selectedItem());
	if (!tBaseItem)return;

	if(tBaseItem->type()==TrollBaseItem::File)
	{
		GroupItem *tGroupItem=(GroupItem*)tBaseItem->parent();
		if(tGroupItem->groupType==GroupItem::InstallObject)
		{
			tGroupItem->installObj()->files.remove(tBaseItem->text(0));
			m_shownSubproject->saveConfig();
		}
		else
		{
			QString aFileName=m_shownSubproject->removeFile(tBaseItem->text(0));
			removeSubclassingInfo(aFileName);
			emitRemovedFile(aFileName);
		}
		m_shownSubproject->showSources(details);
	}
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::removeSubclassingInfo( const QString & aFileName )
{
	//remove subclassing info
	QDomDocument &dom = *(m_part->projectDom());
	DomUtil::PairList list = DomUtil::readPairListEntry(dom,"/kdevtrollproject/subclassing" , "subclass","sourcefile", "uifile");

	QPtrList<DomUtil::Pair> pairsToRemove;

	DomUtil::PairList::iterator it;
	for ( it = list.begin(); it != list.end(); ++it )
	{
		if ( ((*it).first == aFileName) || ((*it).second == aFileName) )
		{
			pairsToRemove.append(&(*it));
		}
	}

	DomUtil::Pair *pair;
	for ( pair = pairsToRemove.first(); pair; pair = pairsToRemove.next() )
	{
		list.remove(*pair);
	}
	QDomElement el = DomUtil::elementByPath( dom,  "/kdevtrollproject");
	QDomElement el2 = DomUtil::elementByPath( dom,  "/kdevtrollproject/subclassing");
	if ( (!el.isNull()) && (!el2.isNull()) )
	{
		el.removeChild(el2);
	}
	DomUtil::writePairListEntry(dom, "/kdevtrollproject/subclassing", "subclass", "sourcefile", "uifile", list);

}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotSubclassWidget( )
{
	TrollBaseItem *tBaseItem=static_cast<TrollBaseItem*>(details->selectedItem());
	if (!tBaseItem)return;

	if(tBaseItem->type()==TrollBaseItem::File)
	{

		QStringList newFileNames;
		newFileNames = m_part->languageSupport()->subclassWidget(m_shownSubproject->path() + "/" + tBaseItem->text(0));
		if (!newFileNames.empty())
		{
			QDomDocument &dom = *(m_part->projectDom());
			for (uint i=0; i<newFileNames.count(); ++i)
			{
				QString srcfile_relpath = newFileNames[i].remove(0,projectDirectory().length());
				QString uifile_relpath = QString(m_shownSubproject->path() + "/" + tBaseItem->text(0)).remove(0,projectDirectory().length());
				DomUtil::PairList list = DomUtil::readPairListEntry(dom,"/kdevtrollproject/subclassing" , "subclass","sourcefile", "uifile");

				list << DomUtil::Pair(srcfile_relpath,uifile_relpath);
				DomUtil::writePairListEntry(dom, "/kdevtrollproject/subclassing", "subclass", "sourcefile", "uifile", list);
				newFileNames[i] = projectDirectory() + newFileNames[i];
				qWarning("new file: %s", newFileNames[i].latin1());
			}
			m_subclasslist = DomUtil::readPairListEntry(dom,"/kdevtrollproject/subclassing" , "subclass","sourcefile", "uifile");

			QStringList tAddedFiles=m_shownSubproject->addFiles(newFileNames);
			m_part->addedFilesToProject(tAddedFiles);
			m_shownSubproject->saveConfig();
			m_shownSubproject->showSources(details);
		}
	}
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotCreateNewFile( )
{
	TrollBaseItem *tBaseItem=static_cast<TrollBaseItem*>(details->selectedItem());
	if (!tBaseItem)return;
	QString tCleanPath = QDir::cleanDirPath(m_shownSubproject->path());
	if(tBaseItem->type()==TrollBaseItem::Group)
	{
		GroupItem* tGroupItem=static_cast<GroupItem*>(details->selectedItem());
		KDevCreateFile * createFileSupport = m_part->createFileSupport();
		if (createFileSupport)
		{
			QString fcext;
			switch (tGroupItem->groupType)
			{
				case GroupItem::Sources:
					fcext = "cpp";
					break;
				case GroupItem::Headers:
					fcext = "h";
					break;
				case GroupItem::Forms:
					fcext = "ui-widget";
					break;
				case GroupItem::Translations:
					fcext = "ts";
					break;
				case GroupItem::Lexsources:
					fcext = "l";
					break;
				case GroupItem::Yaccsources:
					fcext = "y";
					break;
				default:
					fcext = QString::null;
			}
			KDevCreateFile::CreatedFile crFile =  createFileSupport->createNewFile(fcext, tCleanPath);
		}
		else
		{
			bool ok = FALSE;
			QString filename = KLineEditDlg::getText(
			                       i18n( "Create New File"),
			                       i18n( "Enter a name for the new file:" ),
			                       QString::null, &ok, this );
			if ( ok && !filename.isEmpty() )
			{
				QFile newfile(tCleanPath+'/'+filename);
				if (!newfile.open(IO_WriteOnly))
				{
					KMessageBox::error(this,i18n("Failed to create new file. "
					                             "Do you have write permission "
					                             "in the project folder?" ));
					return;
				}
				newfile.close();
				QStringList tFiles;
				tFiles+=filename;
				tFiles=m_shownSubproject->addFiles(tFiles);
				m_shownSubproject->showSources(details);
				emit m_part->addedFilesToProject(tFiles);
			}
		}
	}
	m_shownSubproject->showSources(details);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::addFiles( QStringList & aList, bool noPathTruncate )
{
	QStringList::iterator it;
	for ( it = aList.begin(); it != aList.end(); ++it )
	{
		QFileInfo fi(*it);
		TrollProjectItem * tItem=this->findPrjByPath(QDir::cleanDirPath(fi.dirPath()));
		if(tItem)
		{
			tItem->addFile(fi.fileName());
			tItem->showSources(details);
		}
	}
}


//-------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectItem * TrollProjectWidget::findPrjByPath(const QString &aDir)
{
	QPtrList<TrollProjectItem> tList= m_rootSubproject->getAllProjects();
	QPtrListIterator<TrollProjectItem> it(tList);
	while(*it)
	{
		TrollProjectItem *tItem=(*it);
		if(tItem->getRelativPath()==aDir)return(tItem);
		if(tItem->path()==aDir)return(tItem);
		++it;
	}
	return(NULL);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotMakeCommandProject(TrollProjectItem *aPrjItem,bool autocreateMakefile,bool buildTargetDeps)
{
	QString tTarget;
	QString tPreTargetCmd;
	bool hasSecondTarget=false;

	TrollProjectItem *tItem=m_shownSubproject;
	if(!sender() && !aPrjItem)return;
	if(aPrjItem)tItem=aPrjItem;

	if(sender())
	{
		if(sender()== m_part->actionCollection()->action("build_relink_target"))
		{
			if (m_shownSubproject->isScope())return;
			if (!m_shownSubproject)	return;
			tPreTargetCmd=m_shownSubproject->path()+"/";
			if(!m_shownSubproject->configuration.m_target.isEmpty())
			{
				tPreTargetCmd+=m_shownSubproject->configuration.m_target;
			}
			else
			{
				QFileInfo fi(m_shownSubproject->pro_file);
				tPreTargetCmd+=fi.baseName();
			}

			QFileInfo fi(tPreTargetCmd);
			if(!fi.exists() || !fi.isFile())	return;
			tPreTargetCmd= "rm "+tPreTargetCmd+ " && ";

		}

		if(sender()== m_part->actionCollection()->action("build_compilefile"))
		{
			KParts::ReadWritePart *part = dynamic_cast<KParts::ReadWritePart*>(m_part->partController()->activePart());
			if (!part || !part->url().isLocalFile())	return;
			QFileInfo fi(part->url().path());

			if (!fi.dirPath().startsWith( projectDirectory() ))
			{
				KMessageBox::sorry(this, i18n("Can only compile files in directories which belong to the project."));
				return;
			}

			tItem =  m_rootSubproject->findFile(fi.absFilePath());
			if(!tItem)return;
			if(! tItem->configuration.m_objectpath.isEmpty())
			{
				tTarget=tItem->configuration.m_objectpath;
				tTarget+="/";
			}
			tTarget+=fi.baseName(true) + ".o";
		}


		if(sender()== m_part->actionCollection()->action("build_clean_project"))
		{
			tTarget="clean";
			tItem=m_rootSubproject;
			buildTargetDeps=false;
		}

		if(sender()== m_part->actionCollection()->action("build_distclean_project"))
		{
			tTarget="clean";
			tItem=m_rootSubproject;
			buildTargetDeps=false;
		}

		if(sender()== m_part->actionCollection()->action("build_rebuild_project"))
		{
			tTarget="clean";
			hasSecondTarget=true;
			tItem=m_rootSubproject;
		}
		if(sender()== m_part->actionCollection()->action("build_build_project"))
		{
			tItem=m_rootSubproject;
			buildTargetDeps=false;
		}

		if(sender()== m_part->actionCollection()->action("build_install_project"))
		{
			tTarget="install";
			tItem=m_rootSubproject;
			buildTargetDeps=false;
		}
		
		if(sender()== m_part->actionCollection()->action("build_rebuild_target"))
		{
			tTarget="clean";
			hasSecondTarget=true;
		}

		if(sender()== m_part->actionCollection()->action("build_rebuild_targetonly"))
		{
			tTarget="clean";
			hasSecondTarget=true;
			buildTargetDeps=false;
		}

		if(sender()== m_part->actionCollection()->action("build_build_targetonly"))
		{
			buildTargetDeps=false;
		}

		if(sender()== m_part->actionCollection()->action("build_clean_target")){buildTargetDeps=false;	tTarget="clean";}
		if(sender()== m_part->actionCollection()->action("build_distclean_target"))	{buildTargetDeps=false;tTarget="distclean";}
		if(sender()== m_part->actionCollection()->action("build_mocclean_target"))	{buildTargetDeps=false;tTarget="mocclean";}
		if(sender()== m_part->actionCollection()->action("build_uiclean_target"))	{buildTargetDeps=false;tTarget="uiclean";}
		if(sender()== m_part->actionCollection()->action("build_lexclean_target"))	{buildTargetDeps=false;tTarget="lexclean";}
		if(sender()== m_part->actionCollection()->action("build_yaccclean_target"))	{buildTargetDeps=false;tTarget="yaccclean";}
		if(sender()== m_part->actionCollection()->action("build_install_target"))	{buildTargetDeps=false;tTarget="install";}

		if(sender()== m_part->actionCollection()->action("build_uicables_target"))	tTarget="uicables";
		if(sender()== m_part->actionCollection()->action("build_mocables_target"))	tTarget="mocables";
	}
	m_part->partController()->saveAllFiles();

	if(!tItem)return;
	QString dir = tItem->path();
	createMakefileIfMissing(dir, tItem,!autocreateMakefile);

	if(buildTargetDeps)this->slotMakeDependenyLibs(tItem);



	m_part->mainWindow()->raiseView(m_part->makeFrontend()->widget());
	QString tDirCmd = "cd "+dir + " && " ;
	QString tBuildCmd = constructMakeCommandLine(tItem->configuration.m_makefile)+" "+tTarget;

	if(hasSecondTarget)
	{
		tBuildCmd+=" && ";
		tBuildCmd+=constructMakeCommandLine(tItem->configuration.m_makefile);
	}
	myMakeCommand =tPreTargetCmd+tDirCmd + tBuildCmd;
	m_part->queueCmd(dir,tPreTargetCmd+tDirCmd + tBuildCmd);
	m_part->mainWindow()->lowerView(this);

}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotMakeDependenyLibs( TrollProjectItem * aPrjItem )
{
	if(!aPrjItem)return;
	//FIXME recusion
	
	QPtrList<TrollProjectItem> tCircleList;

	QPtrList<TrollProjectItem> tDeps=aPrjItem->getDepTargets(tCircleList);
	QPtrList<TrollProjectItem> tMakedDeps;
	
	QPtrListIterator<TrollProjectItem> it=(tDeps);
	it.toLast();
	for ( ; (*it); --it )
	{
		// check parents alredy made
		TrollProjectItem *tItem=(*it);
		bool parentMaked=false;
		while(tItem && !parentMaked)
		{
			if(tMakedDeps.find(tItem) >-1)parentMaked=true;
			else tItem=(TrollProjectItem*)tItem->parent();
		}
		
		if((!tMakedDeps.find(*it) > -1) && !parentMaked)
		{
			tMakedDeps.append((*it));
			slotMakeCommandProject((*it),true,false);
			kdDebug(9024) << "make dep target "<<(*it)->text(0)<<endl;
		}
		else
		{
			kdDebug(9024) <<" drop prev maked prj"<< (*it)->text(0);
		}
	}	
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotFileOpenUiHeader( )
{
	TrollBaseItem *tBaseItem=static_cast<TrollBaseItem*>(details->selectedItem());
	if (!tBaseItem)return;

	if(tBaseItem->type()==TrollBaseItem::File)
	{
		m_part->partController()->editDocument(KURL(m_shownSubproject->path() + "/" + QString(tBaseItem->text(0) + ".h")));
	}
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotAddInstallRoot( )
{
	if(!m_shownSubproject)return;
	bool ok = FALSE;
	QString install_obj = KLineEditDlg::getText(
	                          i18n( "Add Install Object" ),
	                          i18n( "Enter a name for the new object:" ),
	                          QString::null, &ok, this );
	if ( ok && !install_obj.isEmpty() )
	{
		if(m_shownSubproject->configuration.addInstallRoot(install_obj))
		{
			m_shownSubproject->showSources(details);
		}
	}
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotSetInstallPath( )
{
	if(!details->selectedItem())return;
	
	TrollBaseItem*tItem=(TrollBaseItem*)details->selectedItem();
	if(!tItem || tItem->type()!=TrollBaseItem::Group)return;
	
	GroupItem *tGroupItem=(GroupItem*)tItem;
	if(!tGroupItem || tGroupItem->groupType!=GroupItem::InstallObject)return;
	
	
	KURLRequesterDlg dialog(i18n( "Choose Install Path" ), i18n( "Enter a path "
	                        "(example /usr/local/share/... ):" ), this, 0);
	dialog.urlRequester()->setMode(KFile::Directory);
	
	dialog.urlRequester()->setURL(tGroupItem->installObj()->install_path);
	if (dialog.exec() == QDialog::Accepted)
	{
		tGroupItem->installObj()->install_path = dialog.urlRequester()->url();
		m_shownSubproject->saveConfig();
	}
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotAddInstallFiles( )
{
	TrollBaseItem*tItem=(TrollBaseItem*)details->selectedItem();
	if(!tItem || tItem->type()!=TrollBaseItem::Group)return;
	GroupItem *tGroupItem=(GroupItem*)tItem;
	if(!tGroupItem || tGroupItem->groupType!=GroupItem::InstallObject)return;
	
	
          bool ok = FALSE;
          QString filepattern = KLineEditDlg::getText(
                              i18n( "Add Pattern of Files to Install" ),
                              i18n( "Enter a pattern relative to the current "
                                    "subproject (example docs/*.html):" ),
                              QString::null, &ok, this );
          if ( ok && !filepattern.isEmpty() )
          {
	  	if(!tGroupItem->installObj()->files.contains(filepattern))
		{
			tGroupItem->installObj()->files.append(filepattern);
			m_shownSubproject->saveConfig();
			m_shownSubproject->showSources(details);
		}
          }
	
/*	
	KURLRequesterDlg dialog(i18n( "Choose Install Path" ), i18n( "Enter a path "
	                        "(example /usr/local/share/... ):" ), this, 0);
	dialog.urlRequester()->setMode(KFile::File);
	dialog.urlRequester()->setURL(tGroupItem->installObj()->install_path);
	if (dialog.exec() == QDialog::Accepted)
	{
		tGroupItem->installObj()->files = dialog.urlRequester()->url();
		m_shownSubproject->saveConfig();
	}
*/
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotRemoveInstallRoot( )
{
	TrollBaseItem*tItem=(TrollBaseItem*)details->selectedItem();
	if(!tItem || tItem->type()!=TrollBaseItem::Group)return;
	GroupItem *tGroupItem=(GroupItem*)tItem;
	if(!tGroupItem || tGroupItem->groupType!=GroupItem::InstallObject)return;
	m_shownSubproject->configuration.removeInstallRoot(tItem->text(0));
	m_shownSubproject->saveConfig();
	m_shownSubproject->showSources(details);

}

void TrollProjectWidget::slotFileDirty( const QString & aFileName)
{
	kdDebug(9024) << "file Dirty "<<aFileName<<endl;
}

void TrollProjectWidget::slotFileDirty( const KURL & aUrl)
{
	kdDebug(9024) << "file Dirty "<<aUrl.fileName()<<endl;

}

void TrollProjectWidget::slotSetMainTarget( )
{
	if(!m_shownSubproject)return;
	if(myMainTarget)myMainTarget->setBold(false);
	m_shownSubproject->setBold(true);
	myMainTarget=m_shownSubproject;
	overview->repaint();
	/*
	QDomDocument &dom = *(m_part->projectDom());

	QString directoryRadioString = DomUtil::readEntry(dom, "/kdevtrollproject/run/directoryradio");
	QString DomMainProgram = DomUtil::readEntry(dom, "/kdevtrollproject/run/mainprogram");

	if ( directoryRadioString == "custom" )
		return DomMainProgram;

	if ( directoryRadioString == "executable" )
	{
		int pos = DomMainProgram.findRev('/');
		if (pos != -1)
			return DomMainProgram.mid(pos+1);

		if ( DomMainProgram.isEmpty() )
		{
			return runDirectory() + "/" + m_widget->getCurrentOutputFilename();
		}
		return DomMainProgram;
	}
	else
		return DomMainProgram;
	*/	
}

//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectWidget::getCurrentOutputFilename()
{
	if (!m_shownSubproject)
		return "";
	if (m_shownSubproject->configuration.m_target.isEmpty())
	{
		QString exe = m_shownSubproject->pro_file;
		return exe.replace(QRegExp("\\.pro$"), "");
	}
	else
		return m_shownSubproject->configuration.m_target;
}

//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectWidget::getCurrentMainTarget()
{
	if(!myMainTarget)return(getCurrentOutputFilename());
	if (myMainTarget->configuration.m_target.isEmpty())
	{
		QString exe = myMainTarget->pro_file;
		return exe.replace(QRegExp("\\.pro$"), "");
	}
	else
		return myMainTarget->configuration.m_target;
}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotCreatePchDirLinks( )
{
	if (!m_shownSubproject)	return;
	
	KURLRequesterDlg dialog("", i18n("Add library directory:"), 0, 0);
	dialog.urlRequester()->setMode(KFile::Directory);
	
	if (dialog.exec() != QDialog::Accepted)	return;
	QString tStr = dialog.urlRequester()->url();
	if (tStr.isEmpty())return;
	QDir tDir(tStr);
	if(!tDir.exists())return;
	
	QListViewItemIterator it(m_shownSubproject);
	for (; it.current(); ++it)
	{
		TrollProjectItem *tItem=(TrollProjectItem*)(*it);
		if(tItem->isScope()) continue;
		if(tItem->configuration.m_template== QTMP_SUBDIRS)continue;
		QDir tDestDir(tItem->getRelativPath()+"/"+tItem->text(0)+".gch");
		if(tDestDir.exists())
		{
			tDestDir.remove(tDestDir.absPath());
		}
		//create symlink 
		QString cmd="ln -s "+tStr+"   "+subprojectDirectory() +tDestDir.absPath()+"\n";
		system(cmd.ascii());
	}

	return;

}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotApplyConfigToAllProjects(int aMode)
{
	if (!m_shownSubproject)	return;
	
	
	QListViewItemIterator it(overview);
	for (; it.current(); ++it)
	{
		TrollProjectItem *tItem=(TrollProjectItem*)(*it);
		if(tItem->isScope()) continue;
		if(tItem->configuration.m_template== QTMP_SUBDIRS)continue;
		
		if(aMode & BUILD_MODE) 		tItem->configuration.m_buildMode= m_shownSubproject->configuration.m_buildMode;
		if(aMode & DEBUG_CXX_FLAGS) 	tItem->configuration.m_cxxflags_debug= m_shownSubproject->configuration.m_cxxflags_debug;
		if(aMode & RELEASE_CXX_FLAGS) 	tItem->configuration.m_cxxflags_release= m_shownSubproject->configuration.m_cxxflags_release;
		if(aMode & CXX_FLAGS) 	tItem->configuration.m_defines= m_shownSubproject->configuration.m_defines;
		if(aMode & LOCATIONS)
		{
		 	tItem->configuration.m_mocpath= m_shownSubproject->configuration.m_mocpath;
		 	tItem->configuration.m_uipath= m_shownSubproject->configuration.m_uipath;
		 	tItem->configuration.m_objectpath= m_shownSubproject->configuration.m_objectpath;
		}
		if(aMode & PCH_INC) 	tItem->configuration.m_pch= m_shownSubproject->configuration.m_pch;
		if(aMode & USE_PCH) 	
		{
			if(m_shownSubproject->configuration.m_requirements & QD_PCH)
				tItem->configuration.m_requirements= tItem->configuration.m_requirements&QD_PCH;
			else
				tItem->configuration.m_requirements= tItem->configuration.m_requirements;
		}
		if(aMode & REQUIREMENTS) 	tItem->configuration.m_requirements= m_shownSubproject->configuration.m_requirements;
		qWarning("before update "+tItem->getRelativPath());
		tItem->saveConfig();
		qWarning("after update "+tItem->getRelativPath());
	}

	return;

}

//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectWidget::slotApplyToSubProjects()
{

	if(m_part->actionCollection()->action("apply_build_mode")==sender())slotApplyConfigToAllProjects(BUILD_MODE);

	if(m_part->actionCollection()->action("apply_debug_cxx")==sender())slotApplyConfigToAllProjects(DEBUG_CXX_FLAGS);
	if(m_part->actionCollection()->action("apply_release_cxx")==sender())slotApplyConfigToAllProjects(RELEASE_CXX_FLAGS);
	if(m_part->actionCollection()->action("apply_cxx")==sender())slotApplyConfigToAllProjects(CXX_FLAGS);
	if(m_part->actionCollection()->action("apply_locations")==sender())slotApplyConfigToAllProjects(LOCATIONS);
	if(m_part->actionCollection()->action("apply_pch_include")==sender())slotApplyConfigToAllProjects(PCH_INC);
	if(m_part->actionCollection()->action("apply_use_pch")==sender())slotApplyConfigToAllProjects(USE_PCH);
	if(m_part->actionCollection()->action("apply_requirements")==sender())slotApplyConfigToAllProjects(REQUIREMENTS);
	if(m_part->actionCollection()->action("apply_all")==sender())slotApplyConfigToAllProjects(BUILD_MODE|DEBUG_CXX_FLAGS|RELEASE_CXX_FLAGS|CXX_FLAGS|LOCATIONS|PCH_INC|USE_PCH|REQUIREMENTS);
}

#include "trollprojectwidget.moc"
