/***************************************************************************
 *   Copyright (C) 2001 by Bernd Gehrmann
 *   bernd@kdevelop.org
 *
 *   Copyright (C) 2000-2001 by Trolltech AS
 *   info@trolltech.com
 *
 *   Copyright (C) 2002 by Jakob Simon-Gaarde
 *   jakob@jsg.dk
 *
 *   Copyright (C) 2002-2003 by Alexander Dymo
 *   cloudtemple@mksat.net
 *
 *   Copyright (C) 2003 - 2004 by Thomas Hasart
 *   thasart@gmx.de
 *
 *   Part of this file is taken from Qt Designer
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version
 *
 ***************************************************************************/

#ifndef _TROLLPROJECTWIDGET_H_
#define _TROLLPROJECTWIDGET_H_

#include <qdict.h>
#include <qlistview.h>
#include <qmap.h>
#include <qstrlist.h>
#include <qvbox.h>
#include <qtoolbutton.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kdeversion.h>
#include <qbutton.h>
#include <qfileinfo.h>
#include <qptrlist.h>
#include "filebuffer.h"
#include "choosesubprojectdlg.h"
#include "filepropertydlg.h"
#include "newwidgetdlg.h"
#include "domutil.h"
#include "trollprojectconfig.h"


#include "trollprojectitem.h"
#include <qguardedptr.h>

enum APPLY_MODE{NONE,BUILD_MODE,DEBUG_CXX_FLAGS,RELEASE_CXX_FLAGS,CXX_FLAGS,LOCATIONS,PCH_INC,USE_PCH,REQUIREMENTS};

class TPConfigDialog;

class TrollProjectWidget : public QVBox
{
		Q_OBJECT

	public:
		TrollProjectWidget(TrollProjectPart *part);
		~TrollProjectWidget();

		void openProject(const QString &dirName);
		void closeProject();

		/**
		 * A list of the (relative) names of all subprojects (== subdirectories).
		 */
		QStringList allSubprojects();
		/**
		 * A list of the (relative) names of all libraries.
		 */
		QStringList allLibraries();
		/**
		 * A list of all files that belong to the project.
		 **/
		QStringList allFiles();
		/**
		 * The top level directory of the project.
		 **/
		QString projectDirectory();
		/**
		 * The directory of the currently active subproject.
		 */
		QString subprojectDirectory();
		/**
		 * The directory of the currently active subproject.
		 */
		QString getCurrentTarget();

		QString getCurrentDestDir();

		QString getCurrentOutputFilename();
		QString getCurrentMainTarget();

		void addFileToCurrentSubProject(GroupItem *titem,const QString &filename);
		void addFileToCurrentSubProject(GroupItem::GroupType gtype,const QString &filename);
		void addFiles( QStringList &files, bool noPathTruncate = false);
		void emitAddedFile(const QString &name);
		void emitRemovedFile(const QString &name);
		

	public slots:
		void init(TrollProjectPart *part);
		void slotExecuteTarget();
		void slotExecuteTarget2();
		void slotExecuteTarget3(const QString& aCmd);
		void slotAutoCompileTargetFailed(const QString&aCmd);
		void slotExecuteProject();
		void slotExecuteProject2();
		void slotExecuteProject3(const QString& aCmd);
		void slotAutoCompileProjectFailed(const QString&aCmd);
		void slotSetMainTarget();

		void slotBuildFile();

		void slotConfigureProject();
		void slotAddFiles();
		void slotRunQMake();
		void slotMakeCommandProject(TrollProjectItem *spitem=NULL,bool autocreateMakefile=false,bool buildTargetDeps=true);
		void slotMakeDependenyLibs(TrollProjectItem *spitem);
		void slotFileOpenUiHeader();
		
		void slotFileDirty(const QString&);
		void slotFileDirty(const KURL&);
		
	private slots:
		void slotOverviewSelectionChanged(QListViewItem *item);
		void slotOverviewContextMenu(KListView *, QListViewItem *item, const QPoint &p);
		void slotDetailsSelectionChanged(QListViewItem*);
		void slotDetailsExecuted(QListViewItem *item);
		void slotDetailsContextMenu(KListView *, QListViewItem *item, const QPoint &p);
		void slotConfigureFile();
		void slotAddSubdir(TrollProjectItem *spitem=0);
		void slotCreateScope(TrollProjectItem *spitem=0);
		void slotRemoveScope(TrollProjectItem *spitem=0);
		void slotBuildModeDebug();
		void slotBuildModeRelease();
		void slotBuildModeDebugAll();
		void slotBuildModeReleaseAll();
		void setBuildMode(QListViewItem *aItem, bool rekursive,QMakeBuildMode aMode);
		void slotConfigFinished();

		void slotRemoveFile();
		void slotSubclassWidget();
		void slotCreateNewFile();
		void slotRemoveSubproject();

		void slotApplyClicked();
		void slotOkClicked();
		void slotCancelClicked();

		void slotAddInstallRoot();
		void slotSetInstallPath();
		void slotAddInstallFiles();
		void slotRemoveInstallRoot();
		void slotCreatePchDirLinks();
		void slotApplyConfigToAllProjects(int aMode);
		void slotApplyToSubProjects();

	private:
		void createPchDirLinks();
		void removeSubclassingInfo(const QString& aFileName);
		void buildProjectDetailTree(TrollProjectItem *item, KListView *listviewControl);
		void removeFile(TrollProjectItem *spitem, FileItem *fitem);
		void setupContext();
		QString getUiFileLink(const QString &path, const QString& filename);
		QString constructMakeCommandLine(const QString makeFileName = QString::null);
		void startMakeCommand(const QString &dir, const QString &target);
		void createMakefileIfMissing(const QString &dir, TrollProjectItem *item,bool ask=true);

		/*fileName: full base file name like QFileInfo::baseName ( true )*/
		TrollProjectItem* findPrjByPath(const QString& aDir);
		void findSubprojectForFile( QPtrList<TrollProjectItem> &list, TrollProjectItem * item, QString absFilePath );
		//    QString makeEnvironment();

		TrollProjectItem *findSubprojectForScope(TrollProjectItem *scope);

		QVBox     *overviewContainer;
		KListView *overview;
		QHBox     *projectTools;



		//QToolButton *projectconfButton;

		QVBox     *detailContainer;
		KListView *details;
		QHBox     *fileTools;

		DomUtil::PairList m_subclasslist;
		TrollProjectItem *m_shownSubproject;
		TrollProjectItem *m_rootSubproject;
		TrollProjectPart *m_part;
		TPConfigDialog *myConfigDialog;

		friend class ChooseSubprojectDlg;
		
		QString myMakeCommand;
		TrollProjectItem* myMainTarget;
};

#endif
